@@ -5,8 +5,4 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
-__version__ = "0.7.0"
-
-from .base.application import BaseApplication
-from .base.plot import PlotSelection2D
-from .base.selection import ObjectDataSelection
+__version__ = "0.9.0-post.1"
