@@ -5,34 +5,49 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 import json
 import multiprocessing
 import os
 
-import ipywidgets as widgets
-import matplotlib.pyplot as plt
 import numpy as np
+from geoh5py.data import Data
 from geoh5py.groups import ContainerGroup
-from geoh5py.objects import BlockModel, Curve, Octree, Points, Surface
-from geoh5py.workspace import Workspace
-from ipywidgets.widgets import (
-    Button,
-    Checkbox,
-    Dropdown,
-    FloatText,
-    HBox,
-    IntText,
-    Label,
-    Layout,
-    Text,
-    VBox,
+from geoh5py.objects import (
+    AirborneTEMReceivers,
+    BlockModel,
+    Curve,
+    Octree,
+    Points,
+    Surface,
 )
+from geoh5py.workspace import Workspace
 
-from geoapps import PlotSelection2D
 from geoapps.base.application import BaseApplication
+from geoapps.base.plot import PlotSelection2D
 from geoapps.base.selection import LineOptions, ObjectDataSelection, TopographyOptions
-from geoapps.utils import geophysical_systems
-from geoapps.utils.utils import find_value, string_2_list
+from geoapps.utils import geophysical_systems, warn_module_not_found
+from geoapps.utils.list import find_value
+from geoapps.utils.string import string_2_list
+
+with warn_module_not_found():
+    from matplotlib import pyplot as plt
+
+with warn_module_not_found():
+    import ipywidgets as widgets
+    from ipywidgets.widgets import (
+        Button,
+        Checkbox,
+        Dropdown,
+        FloatText,
+        HBox,
+        IntText,
+        Label,
+        Layout,
+        Text,
+        VBox,
+    )
 
 
 class ChannelOptions:
@@ -70,7 +85,7 @@ class ChannelOptions:
             if hasattr(self, "_" + key):
                 try:
                     getattr(self, key).value = value
-                except:
+                except AttributeError:
                     pass
 
     @property
@@ -103,7 +118,6 @@ class SensorOptions(ObjectDataSelection):
     Define the receiver spatial parameters
     """
 
-    defaults = {}
     _options = None
 
     def __init__(self, **kwargs):
@@ -112,7 +126,7 @@ class SensorOptions(ObjectDataSelection):
         self._constant = FloatText(
             description="Constant elevation (m)",
         )
-        if "offset" in self.defaults.keys():
+        if "offset" in self.defaults:
             self._offset.value = self.defaults["offset"]
 
         self.option_list = {
@@ -215,7 +229,7 @@ class MeshOctreeOptions:
             if hasattr(self, "_" + key):
                 try:
                     getattr(self, key).value = value
-                except:
+                except AttributeError:
                     pass
 
     @property
@@ -287,7 +301,7 @@ class Mesh1DOptions:
             if hasattr(self, "_" + key):
                 try:
                     getattr(self, key).value = value
-                except:
+                except AttributeError:
                     pass
 
     def count_cells(self):
@@ -384,8 +398,6 @@ class InversionOptions(BaseApplication):
     Collection of widgets controlling the inversion parameters
     """
 
-    defaults = {}
-
     def __init__(self, **kwargs):
         self.defaults.update(**kwargs)
         self._chi_factor = FloatText(
@@ -477,8 +489,8 @@ class InversionOptions(BaseApplication):
             "optimization": self._optimization,
         }
         self.option_choices = widgets.Dropdown(
-            options=list(self.inversion_options.keys()),
-            value=list(self.inversion_options.keys())[0],
+            options=list(self.inversion_options),
+            value=list(self.inversion_options)[0],
             disabled=False,
         )
         self.option_choices.observe(self.inversion_option_change, names="value")
@@ -689,37 +701,43 @@ def inversion_defaults():
     return defaults
 
 
-class InversionApp(PlotSelection2D):
-    defaults = {
-        "h5file": "../../../assets/FlinFlon.geoh5",
-        "objects": "{656acd40-25de-4865-814c-cb700f6ee51a}",
-        "data": ["{2d165431-63bd-4e07-9db8-5b44acf8c9bf}"],
-        "resolution": 50,
-        "window_width": 1500,
-        "window_height": 1500,
-        "ga_group_name": "EM1DInversion_",
-        "inversion_parameters": {"max_iterations": 25},
-        "topography": {
-            "options": "Object",
-            "objects": "{ab3c2083-6ea8-4d31-9230-7aad3ec09525}",
-            "data": "{a603a762-f6cb-4b21-afda-3160e725bf7d}",
-        },
-        "lines": {
-            "data": "{c004eda2-70f6-4181-9a55-e7b2e1b25da8}",
-            "lines": [6072400.0, 6072600.0, 6072800.0, 6073000.0, 6073200.0],
-        },
-        "sensor": {"offset": "0, 0, 0", "options": "sensor location + (dx, dy, dz)"},
-    }
+app_initializer = {
+    "geoh5": "../../../assets/FlinFlon.geoh5",
+    "objects": "{656acd40-25de-4865-814c-cb700f6ee51a}",
+    "data": "{2d165431-63bd-4e07-9db8-5b44acf8c9bf}",
+    "resolution": 50,
+    "window_width": 1500,
+    "window_height": 1500,
+    "ga_group_name": "EM1DInversion_",
+    "inversion_parameters": {"max_iterations": 25},
+    "topography": {
+        "options": "Object",
+        "objects": "{ab3c2083-6ea8-4d31-9230-7aad3ec09525}",
+        "data": "{a603a762-f6cb-4b21-afda-3160e725bf7d}",
+    },
+    "lines": {
+        "data": "{b6927372-22d2-4ca1-9c23-372411ddc772}",
+        "lines": [8, 9, 10, 11, 12],
+    },
+    "sensor": {"offset": "0, 0, 0", "options": "sensor location + (dx, dy, dz)"},
+}
 
-    _select_multiple = True
-    _add_groups = True
+
+class InversionApp(PlotSelection2D):
+
+    _select_multiple = False
+    _add_groups = "only"
     _sensor = None
     _lines = None
     _topography = None
     _inversion_parameters = None
 
     def __init__(self, **kwargs):
+        self.defaults.update(**app_initializer)
         self.defaults.update(**kwargs)
+        self._uncertainties = Dropdown(
+            description="Uncertainties: ",
+        )
         self.em_system_specs = geophysical_systems.parameters()
         self._data_count = (Label("Data Count: 0"),)
         self._forward_only = Checkbox(
@@ -728,7 +746,7 @@ class InversionApp(PlotSelection2D):
         )
         self._starting_channel = (IntText(value=None, description="Starting Channel"),)
         self._system = Dropdown(
-            options=list(self.em_system_specs.keys()),
+            options=["Airborne TEM Survey"] + list(self.em_system_specs),
             description="Survey Type: ",
         )
         self._write = Button(
@@ -749,10 +767,10 @@ class InversionApp(PlotSelection2D):
         self.data_channel_choices.observe(
             self.data_channel_choices_observer, names="value"
         )
-
+        self.plotting_data = None
         super().__init__(**self.defaults)
 
-        if "lines" in self.defaults:
+        if "lines" in app_initializer:
             self.lines.__populate__(**self.defaults["lines"])
 
         self.output_panel = VBox([self.export_directory, self.write, self.trigger])
@@ -795,7 +813,7 @@ class InversionApp(PlotSelection2D):
                     self.project_panel,
                     HBox(
                         [
-                            self.data_panel,
+                            VBox([self.objects, self.data, self._uncertainties]),
                             VBox(
                                 [
                                     VBox(
@@ -935,30 +953,15 @@ class InversionApp(PlotSelection2D):
         """
         Change the application on change of system
         """
-        tx_offsets = self.em_system_specs[self.system.value]["tx_offsets"]
+
         self.sensor.offset.value = ", ".join(
             [
                 str(offset)
                 for offset in self.em_system_specs[self.system.value]["bird_offset"]
             ]
         )
-        uncertainties = self.em_system_specs[self.system.value]["uncertainty"]
 
         # if start_channel is None:
-        start_channel = self.em_system_specs[self.system.value]["channel_start_index"]
-        if self.em_system_specs[self.system.value]["type"] == "time":
-            labels = ["Time (s)"] * len(
-                self.em_system_specs[self.system.value]["channels"]
-            )
-        else:
-            labels = ["Frequency (Hz)"] * len(
-                self.em_system_specs[self.system.value]["channels"]
-            )
-
-        system_specs = {}
-        for key, time in self.em_system_specs[self.system.value]["channels"].items():
-            system_specs[key] = f"{time:.5e}"
-
         self.inversion_parameters.reference_model.options.options = [
             "Best-fitting halfspace",
             "Model",
@@ -972,7 +975,9 @@ class InversionApp(PlotSelection2D):
         self.inversion_parameters.ignore_values.value = "<0"
 
         # Switch mesh options
-        self.inversion_parameters._mesh = self.mesh_1D
+        self.inversion_parameters._mesh = (  # pylint: disable=protected-access
+            self.mesh_1D
+        )
         self.inversion_parameters.inversion_options["mesh"] = self.mesh_1D.main
         flag = "EM1D"
 
@@ -1001,6 +1006,9 @@ class InversionApp(PlotSelection2D):
             data_widget = self.data_channel_choices.data_channel_options[channel.header]
 
             entity = self.workspace.get_entity(self.objects.value)[0]
+            if entity is None:
+                return
+
             if channel.value is None or channel.value not in [
                 child.uid for child in entity.children
             ]:
@@ -1014,50 +1022,75 @@ class InversionApp(PlotSelection2D):
                 self.refresh.value = True
 
         data_channel_options = {}
-        for ind, (key, channel) in enumerate(system_specs.items()):
-            if ind + 1 < start_channel:
-                continue
-
-            if len(tx_offsets) > 1:
-                offsets = tx_offsets[ind]
+        if self.system.value != "Airborne TEM Survey":
+            if self.em_system_specs[self.system.value]["type"] == "time":
+                labels = ["Time (s)"] * len(
+                    self.em_system_specs[self.system.value]["channels"]
+                )
             else:
-                offsets = tx_offsets[0]
-
-            try:
-                channel_options = ChannelOptions(
-                    key,
-                    labels[ind],
-                    uncertainties=", ".join(
-                        [str(uncert) for uncert in uncertainties[ind][:2]]
-                    ),
-                    offsets=", ".join([str(offset) for offset in offsets]),
+                labels = ["Frequency (Hz)"] * len(
+                    self.em_system_specs[self.system.value]["channels"]
                 )
 
-            except IndexError:
-                continue
-            channel_options.channel_selection.observe(channel_setter, names="value")
-            data_channel_options[key] = channel_options.main
-            data_channel_options[key].children[1].value = channel
+            start_channel = self.em_system_specs[self.system.value][
+                "channel_start_index"
+            ]
+            tx_offsets = self.em_system_specs[self.system.value]["tx_offsets"]
+            uncertainties = self.em_system_specs[self.system.value]["uncertainty"]
+            system_specs = {}
+            for key, time in self.em_system_specs[self.system.value][
+                "channels"
+            ].items():
+                system_specs[key] = f"{time:.5e}"
+
+            for ind, (key, channel) in enumerate(system_specs.items()):
+                if ind + 1 < start_channel:
+                    continue
+
+                if len(tx_offsets) > 1:
+                    offsets = tx_offsets[ind]
+                else:
+                    offsets = tx_offsets[0]
+
+                try:
+                    channel_options = ChannelOptions(
+                        key,
+                        labels[ind],
+                        uncertainties=", ".join(
+                            [str(uncert) for uncert in uncertainties[ind][:2]]
+                        ),
+                        offsets=", ".join([str(offset) for offset in offsets]),
+                    )
+
+                except IndexError:
+                    continue
+                channel_options.channel_selection.observe(channel_setter, names="value")
+                data_channel_options[key] = channel_options.main
+                data_channel_options[key].children[1].value = channel
 
         if len(data_channel_options) > 0:
-            self.data_channel_choices.options = list(data_channel_options.keys())
-            self.data_channel_choices.value = list(data_channel_options.keys())[0]
+            self.data_channel_choices.options = list(data_channel_options)
+            self.data_channel_choices.value = list(data_channel_options)[0]
             self.data_channel_choices.data_channel_options = data_channel_options
             self.data_channel_panel.children = [
                 self.data_channel_choices,
                 data_channel_options[self.data_channel_choices.value],
             ]
+        else:
+            self.data_channel_choices.options = []
+            self.data_channel_choices.data_channel_options = None
+            self.data_channel_panel.children = [self.data_channel_choices]
 
         self.update_component_panel(None)
 
         if self.em_system_specs[self.system.value]["type"] == "frequency":
             self.inversion_parameters.option_choices.options = list(
-                self.inversion_parameters.inversion_options.keys()
+                self.inversion_parameters.inversion_options
             )
         else:
             self.inversion_parameters.option_choices.options = [
                 key
-                for key in self.inversion_parameters.inversion_options.keys()
+                for key in self.inversion_parameters.inversion_options
                 if key != "background susceptibility"
             ]
 
@@ -1067,53 +1100,65 @@ class InversionApp(PlotSelection2D):
     def object_observer(self, _):
 
         self.resolution.indices = None
+        entity = self._workspace.get_entity(self.objects.value)[0]
+        if entity is None:
+            return
 
-        if self.workspace.get_entity(self.objects.value):
-            self.update_data_list(None)
-            self.sensor.update_data_list(None)
-            self.lines.update_data_list(None)
-            self.lines.update_line_list(None)
+        self.update_data_list(None)
+        self.sensor.update_data_list(None)
+        self.lines.update_data_list(None)
+        self.lines.update_line_list(None)
 
+        if isinstance(entity, AirborneTEMReceivers):
+            self.system.value = "Airborne TEM Survey"
+        else:
             for aem_system, specs in self.em_system_specs.items():
-                if any(
-                    [
-                        specs["flag"] in channel
-                        for channel in self.data.uid_name_map.values()
-                    ]
-                ):
+                if aem_system == "Airborne TEM Survey":
+                    continue
+
+                if any([specs["flag"] in data.name for data in entity.children]):
                     self.system.value = aem_system
+                    break
 
-            self.system_observer(None)
+        self.system_observer(None)
 
-            if hasattr(self.data_channel_choices, "data_channel_options"):
-                for (
-                    key,
-                    data_widget,
-                ) in self.data_channel_choices.data_channel_options.items():
-                    data_widget.children[2].options = self.data.options
-                    value = find_value(self.data.options, [key])
-                    data_widget.children[2].value = value
+        if getattr(self.data_channel_choices, "data_channel_options", None) is not None:
+            for (
+                key,
+                data_widget,
+            ) in self.data_channel_choices.data_channel_options.items():
+                data_widget.children[2].options = self.data.options
+                value = find_value(self.data.options, [key])
+                data_widget.children[2].value = value
 
-            self.write.button_style = "warning"
-            self.trigger.button_style = "danger"
+        self.write.button_style = "warning"
+        self.trigger.button_style = "danger"
 
     def update_component_panel(self, _):
-        if self.workspace.get_entity(self.objects.value):
-            _, data_list = self.get_selected_entities()
-            options = [[data.name, data.uid] for data in data_list]
-            if hasattr(self.data_channel_choices, "data_channel_options"):
-                for (
-                    key,
-                    data_widget,
-                ) in self.data_channel_choices.data_channel_options.items():
-                    data_widget.children[2].options = options
-                    data_widget.children[2].value = find_value(options, [key])
+
+        obj, data_list = self.get_selected_entities()
+
+        if obj is None:
+            return
+
+        options = [
+            [data.name, data.uid] for data in data_list if isinstance(data, Data)
+        ]
+
+        self._uncertainties.options = self.data.options
+        if getattr(self.data_channel_choices, "data_channel_options", None) is not None:
+            for (
+                key,
+                data_widget,
+            ) in self.data_channel_choices.data_channel_options.items():
+                data_widget.children[2].options = options
+                data_widget.children[2].value = find_value(options, [key])
 
     def data_channel_choices_observer(self, _):
-        if hasattr(
-            self.data_channel_choices, "data_channel_options"
-        ) and self.data_channel_choices.value in (
-            self.data_channel_choices.data_channel_options.keys()
+        if getattr(
+            self.data_channel_choices, "data_channel_options", None
+        ) is not None and self.data_channel_choices.value in (
+            self.data_channel_choices.data_channel_options
         ):
             data_widget = self.data_channel_choices.data_channel_options[
                 self.data_channel_choices.value
@@ -1121,7 +1166,7 @@ class InversionApp(PlotSelection2D):
             self.data_channel_panel.children = [self.data_channel_choices, data_widget]
 
             if (
-                self.workspace.get_entity(self.objects.value)
+                self.workspace.get_entity(self.objects.value)[0] is not None
                 and data_widget.children[2].value is None
             ):
                 _, data_list = self.get_selected_entities()
@@ -1146,26 +1191,20 @@ class InversionApp(PlotSelection2D):
         input_dict = {
             "out_group": self.ga_group_name.value,
             "mesh": "Mesh",
+            "system": self.system.value,
+            "lines": {str(self.lines.data.value): [self.lines.lines.value]},
+            "mesh 1D": [
+                self.mesh_1D.hz_min.value,
+                self.mesh_1D.hz_expansion.value,
+                self.mesh_1D.n_cells.value,
+            ],
+            "uncertainty_mode": self.inversion_parameters.uncert_mode.value,
+            "chi_factor": self.inversion_parameters.chi_factor.value,
+            "max_iterations": self.inversion_parameters.max_iterations.value,
+            "max_cg_iterations": self.inversion_parameters.max_cg_iterations.value,
+            "n_cpu": self.inversion_parameters.n_cpu.value,
+            "max_ram": self.inversion_parameters.max_ram.value,
         }
-        input_dict["system"] = self.system.value
-        input_dict["lines"] = {
-            str(self.lines.data.value): [str(line) for line in self.lines.lines.value]
-        }
-
-        input_dict["mesh 1D"] = [
-            self.mesh_1D.hz_min.value,
-            self.mesh_1D.hz_expansion.value,
-            self.mesh_1D.n_cells.value,
-        ]
-        input_dict["uncertainty_mode"] = self.inversion_parameters.uncert_mode.value
-
-        input_dict["chi_factor"] = self.inversion_parameters.chi_factor.value
-        input_dict["max_iterations"] = self.inversion_parameters.max_iterations.value
-        input_dict[
-            "max_cg_iterations"
-        ] = self.inversion_parameters.max_cg_iterations.value
-        input_dict["n_cpu"] = self.inversion_parameters.n_cpu.value
-        input_dict["max_ram"] = self.inversion_parameters.max_ram.value
 
         if self.inversion_parameters.beta_start_options.value == "value":
             input_dict["initial_beta"] = self.inversion_parameters.beta_start.value
@@ -1252,11 +1291,9 @@ class InversionApp(PlotSelection2D):
                 self.inversion_parameters.upper_bound.value
             )
 
-        input_dict["data"] = {}
-        input_dict["data"]["type"] = "GA_object"
-        input_dict["data"]["name"] = str(self.objects.value)
+        input_dict["data"] = {"type": "GA_object", "name": str(self.objects.value)}
 
-        if hasattr(self.data_channel_choices, "data_channel_options"):
+        if getattr(self.data_channel_choices, "data_channel_options", None) is not None:
             channel_param = {}
 
             for (
@@ -1278,6 +1315,8 @@ class InversionApp(PlotSelection2D):
                 )
 
             input_dict["data"]["channels"] = channel_param
+        else:
+            input_dict["data"]["channels"] = str(self.data.value)
 
         if self.sensor.options.value == "sensor location + (dx, dy, dz)":
             input_dict["receivers_offset"] = {
@@ -1309,7 +1348,7 @@ class InversionApp(PlotSelection2D):
 
         checks = [key for key, val in input_dict.items() if val is None]
 
-        if len(list(input_dict["data"]["channels"].keys())) == 0:
+        if len(list(input_dict["data"]["channels"])) == 0:
             checks += ["'Channel' for at least one data component."]
 
         if len(checks) > 0:
@@ -1317,44 +1356,89 @@ class InversionApp(PlotSelection2D):
             self.trigger.button_style = "danger"
             return
 
-        new_workspace = Workspace(
+        if (
+            self.system.value == "Airborne TEM Survey"
+            and self._uncertainties.value is None
+        ):
+            print("TEM survey requires an Uncertainty group.")
+            return
+
+        with Workspace(
             os.path.join(
                 self.export_directory.selected_path,
                 self.ga_group_name.value + ".geoh5",
             )
-        )
+        ) as new_workspace:
 
-        obj, data = self.get_selected_entities()
-        new_obj = obj.copy(parent=new_workspace, copy_children=False)
-        for d in data:
-            d.copy(parent=new_obj)
+            obj, _ = self.get_selected_entities()
+            new_obj = obj.copy(parent=new_workspace, copy_children=False)
 
-        _, data = self.sensor.get_selected_entities()
-        for d in data:
-            d.copy(parent=new_obj)
+            prop_group = obj.find_or_create_property_group(
+                name=self.data.uid_name_map[self.data.value]
+            )
+            data_list = []
+            for prop in prop_group.properties:
+                data = self.workspace.get_entity(prop)[0]
+                data_list.append(data.copy(parent=new_obj))
 
-        _, data = self.lines.get_selected_entities()
-        for d in data:
-            d.copy(parent=new_obj)
+            new_group = new_obj.add_data_to_group(data_list, prop_group.name)
 
-        for elem in [
-            self.topography,
-            self.inversion_parameters.starting_model,
-            self.inversion_parameters.reference_model,
-            self.inversion_parameters.susceptibility_model,
-        ]:
-            obj, data = elem.get_selected_entities()
+            if isinstance(input_dict["data"]["channels"], str):
+                input_dict["data"]["channels"] = str(new_group.uid)
 
-            if obj is not None:
-                new_obj = obj.copy(parent=new_workspace, copy_children=False)
-                for d in data:
-                    d.copy(parent=new_obj)
+            if self._uncertainties.value is not None:
+
+                prop_group = obj.find_or_create_property_group(
+                    name=self.data.uid_name_map[self._uncertainties.value]
+                )
+                data_list = []
+                for prop in prop_group.properties:
+                    data = self.workspace.get_entity(prop)[0]
+                    data_list.append(data.copy(parent=new_obj))
+
+                new_group = new_obj.add_data_to_group(data_list, prop_group.name)
+                input_dict["uncertainty_channel"] = str(new_group.uid)
+            # if self.system.value == "Airborne TEM Survey":
+            #     prop_group = new_obj.add_data_to_group(data_list, )
+            #
+
+            _, data = self.sensor.get_selected_entities()
+            for d in data:
+                if d is None:
+                    continue
+                d.copy(parent=new_obj)
+
+            _, data = self.lines.get_selected_entities()
+            for d in data:
+                if d is None:
+                    continue
+                d.copy(parent=new_obj)
+
+            for elem in [
+                self.topography,
+                self.inversion_parameters.starting_model,
+                self.inversion_parameters.reference_model,
+                self.inversion_parameters.susceptibility_model,
+            ]:
+                obj, data = elem.get_selected_entities()
+
+                if obj is not None:
+
+                    new_obj = new_workspace.get_entity(obj.uid)[0]
+                    if new_obj is None:
+                        new_obj = obj.copy(parent=new_workspace, copy_children=False)
+
+                    for d in data:
+                        if d is None:
+                            continue
+                        d.copy(parent=new_obj)
 
         input_dict["workspace"] = os.path.abspath(new_workspace.h5file)
         input_dict["save_to_geoh5"] = os.path.abspath(new_workspace.h5file)
 
         file = f"{os.path.join(self.export_directory.selected_path, self.ga_group_name.value)}.json"
-        with open(file, "w") as f:
+
+        with open(file, "w", encoding="utf8") as f:
             json.dump(input_dict, f, indent=4)
 
         self.write.button_style = ""
