@@ -5,6 +5,9 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+# pylint: disable=W0221
+# pylint: disable=W0622
+
 from __future__ import annotations
 
 from typing import TYPE_CHECKING, Any
@@ -21,7 +24,8 @@ from SimPEG import maps
 from SimPEG.electromagnetics.static.utils.static_utils import geometric_factor
 from SimPEG.utils.drivers import create_nested_mesh
 
-from geoapps.utils import calculate_2D_trend, filter_xy
+from geoapps.inversion.utils import calculate_2D_trend
+from geoapps.shared_utils.utils import filter_xy
 
 from .factories import (
     EntityFactory,
@@ -104,6 +108,7 @@ class InversionData(InversionLocations):
         self.locations: np.ndarray = None
         self.has_pseudo: bool = False
         self.mask: np.ndarray = None
+        self.global_map: np.ndarray = None
         self.indices: np.ndarray = None
         self.vector: bool = None
         self.n_blocks: int = None
@@ -125,6 +130,7 @@ class InversionData(InversionLocations):
         self.n_blocks = 3 if self.params.inversion_type == "magnetic vector" else 1
         self.ignore_value, self.ignore_type = self.parse_ignore_values()
         self.components, self.observed, self.uncertainties = self.get_data()
+        self.has_tensor = InversionData.check_tensor(self.components)
         self.offset, self.radar = self.params.offset()
         self.locations = super().get_locations(self.params.data_object)
         self.mask = filter_xy(
@@ -160,7 +166,13 @@ class InversionData(InversionLocations):
     def filter(self, a):
         """Remove vertices based on mask property."""
         if (
-            self.params.inversion_type in ["direct current", "induced polarization"]
+            self.params.inversion_type
+            in [
+                "direct current",
+                "direct current 2d",
+                "induced polarization",
+                "induced polarization 2d",
+            ]
             and self.indices is None
         ):
             ab_ind = np.where(np.any(self.mask[self.params.data_object.cells], axis=1))[
@@ -238,8 +250,10 @@ class InversionData(InversionLocations):
                             uncert_entity, f"Uncertainties_{component}"
                         )
         else:
-            for component in data.keys():
+            for component in data:
                 dnorm = self.normalizations[component] * data[component]
+                if "2d" in self.params.inversion_type:
+                    dnorm = self._embed_2d(dnorm)
                 data_entity[component] = entity.add_data(
                     {f"{basename}_{component}": {"values": dnorm}}
                 )
@@ -249,15 +263,25 @@ class InversionData(InversionLocations):
                     ].entity_type
                     uncerts = self.uncertainties[component].copy()
                     uncerts[np.isinf(uncerts)] = np.nan
+                    if "2d" in self.params.inversion_type:
+                        uncerts = self._embed_2d(uncerts)
                     entity.add_data({f"Uncertainties_{component}": {"values": uncerts}})
 
-                if self.params.inversion_type == "direct current":
+                if self.params.inversion_type in [
+                    "direct current",
+                    "direct current 2d",
+                ]:
                     self.transformations[component] = 1 / (
                         geometric_factor(self._survey) + 1e-10
                     )
-                    apparent_property = (
-                        data[component] * self.transformations[component]
-                    )
+
+                    apparent_property = data[component].copy()
+                    apparent_property[self.global_map] *= self.transformations[
+                        component
+                    ]
+                    if "2d" in self.params.inversion_type:
+                        apparent_property = self._embed_2d(apparent_property)
+
                     data_entity["apparent_resistivity"] = entity.add_data(
                         {
                             f"{basename}_apparent_resistivity": {
@@ -392,13 +416,10 @@ class InversionData(InversionLocations):
             if comp in ["gz", "bz", "gxz", "gyz", "bxz", "byz"]:
                 normalizations[comp] = -1.0
             elif self.params.inversion_type in ["magnetotellurics"]:
+                normalizations[comp] = -1.0
+            elif self.params.inversion_type in ["tipper"]:
                 if "imag" in comp:
                     normalizations[comp] = -1.0
-            elif self.params.inversion_type in ["tipper"]:
-                if "real" in comp:
-                    normalizations[comp] = -1.0
-            if normalizations[comp] == -1.0:
-                print(f"Sign flip for component {comp}.")
 
         return normalizations
 
@@ -456,10 +477,13 @@ class InversionData(InversionLocations):
         """
         simulation_factory = SimulationFactory(self.params)
 
-        if tile_id is None:
-            map = maps.IdentityMap(nP=int(self.n_blocks * active_cells.sum()))
+        if tile_id is None or "2d" in self.params.inversion_type:
+            mapping = maps.IdentityMap(nP=int(self.n_blocks * active_cells.sum()))
             sim = simulation_factory.build(
-                survey=survey, global_mesh=mesh, active_cells=active_cells, map=map
+                survey=survey,
+                global_mesh=mesh,
+                active_cells=active_cells,
+                mapping=mapping,
             )
 
         else:
@@ -472,18 +496,18 @@ class InversionData(InversionLocations):
             )
 
             kwargs = {"components": 3} if self.vector else {}
-            map = maps.TileMap(
+            mapping = maps.TileMap(
                 mesh, active_cells, nested_mesh, enforce_active=True, **kwargs
             )
             sim = simulation_factory.build(
                 survey=survey,
                 global_mesh=mesh,
                 local_mesh=nested_mesh,
-                active_cells=map.local_active,
-                map=map,
+                active_cells=mapping.local_active,
+                mapping=mapping,
                 tile_id=tile_id,
             )
-        return sim, map
+        return sim, mapping
 
     def simulate(self, model, inverse_problem, sorting):
         """Simulate fields for a particular model."""
@@ -496,3 +520,23 @@ class InversionData(InversionLocations):
                 sorting=np.argsort(np.hstack(sorting)),
             )
             save_directive.save_components(0, dpred)
+
+    @property
+    def observed_data_types(self):
+        """
+        Stored data types
+        """
+        return self._observed_data_types
+
+    def _embed_2d(self, data):
+        ind = np.ones_like(data, dtype=bool)
+        ind[self.global_map] = False
+        data[ind] = np.nan
+        return data
+
+    @staticmethod
+    def check_tensor(channels):
+
+        tensor_components = ["xx", "xy", "xz", "yx", "zx", "yy", "zz", "zy", "yz"]
+        has_tensor = lambda c: any(k in c for k in tensor_components)
+        return any(has_tensor(c) for c in channels)
