@@ -5,6 +5,8 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
 
 
@@ -24,5 +26,5 @@ class AbstractFactory(ABC):
         """Returns a class to be constructed by the build method."""
 
     @abstractmethod
-    def build(self):
+    def build(self, *args):
         """Constructs concrete object for provided factory type."""
