@@ -5,196 +5,18 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 from uuid import UUID
 
-from geoh5py.groups import ContainerGroup
 from geoh5py.objects import Curve, Grid2D, Points, Surface
 
-octree_defaults = {
-    "title": "Inversion mesh creator",
-    "geoh5": None,
-    "objects": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "depth_core": 500.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
-    "Refinement A object": None,
-    "Refinement A levels": [4, 4, 4, 4],
-    "Refinement A type": "radial",
-    "Refinement A distance": 5000.0,
-    "Refinement B object": None,
-    "Refinement B levels": [0, 0, 4, 4],
-    "Refinement B type": "surface",
-    "Refinement B distance": 5000.0,
-    "ga_group_name": "Inversion Mesh",
-    "run_command": "geoapps.octree_creation.application",
-    "run_command_boolean": False,
-    "monitoring_directory": None,
-    "workspace_geoh5": None,
-    "conda_environment": "geoapps",
-    "conda_environment_boolean": False,
-}
-
-
-default_octree_ui_json = {
-    "title": None,
-    "geoh5": None,
-    "objects": {
-        "enabled": True,
-        "group": "Mesh",
-        "label": "Core hull extent",
-        "main": True,
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-        ],
-        "value": None,
-    },
-    "u_cell_size": {
-        "min": 0.0,
-        "group": "Mesh",
-        "main": True,
-        "enabled": True,
-        "dependency": "mesh",
-        "dependencyType": "disabled",
-        "label": "Easting core cell size (m)",
-        "value": 25.0,
-    },
-    "v_cell_size": {
-        "min": 0.0,
-        "group": "Mesh",
-        "main": True,
-        "enabled": True,
-        "dependency": "mesh",
-        "dependencyType": "disabled",
-        "label": "Northing core cell size (m)",
-        "value": 25.0,
-    },
-    "w_cell_size": {
-        "min": 0.0,
-        "group": "Mesh",
-        "main": True,
-        "enabled": True,
-        "dependency": "mesh",
-        "dependencyType": "disabled",
-        "label": "Vertical core cell size (m)",
-        "value": 25.0,
-    },
-    "depth_core": {
-        "min": 0.0,
-        "group": "Mesh",
-        "main": True,
-        "enabled": True,
-        "dependency": "mesh",
-        "dependencyType": "disabled",
-        "label": "Depth of core refinement volume",
-        "value": 500.0,
-    },
-    "horizontal_padding": {
-        "min": 0.0,
-        "group": "Mesh",
-        "main": True,
-        "enabled": True,
-        "dependency": "mesh",
-        "dependencyType": "disabled",
-        "label": "Horizontal padding",
-        "value": 1000.0,
-    },
-    "vertical_padding": {
-        "min": 0.0,
-        "group": "Mesh",
-        "main": True,
-        "enabled": True,
-        "dependency": "mesh",
-        "dependencyType": "disabled",
-        "label": "Vertical padding",
-        "value": 1000.0,
-    },
-    "Refinement A object": {
-        "groupOptional": True,
-        "enabled": True,
-        "group": "Data refinement",
-        "label": "Data object",
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-        ],
-        "value": None,
-    },
-    "Refinement A levels": {
-        "enabled": True,
-        "group": "Data refinement",
-        "label": "Levels",
-        "value": "4, 4, 4, 4",
-    },
-    "Refinement A type": {
-        "choiceList": ["surface", "radial"],
-        "enabled": True,
-        "group": "Data refinement",
-        "label": "Type",
-        "value": "radial",
-    },
-    "Refinement A distance": {
-        "enabled": True,
-        "group": "Data refinement",
-        "label": "Distance",
-        "value": 5000.0,
-    },
-    "Refinement B object": {
-        "groupOptional": True,
-        "enabled": True,
-        "group": "Topography refinement",
-        "label": "Topography object",
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-        ],
-        "value": None,
-    },
-    "Refinement B levels": {
-        "enabled": True,
-        "group": "Topography refinement",
-        "label": "Levels",
-        "value": "0, 0, 4, 4",
-    },
-    "Refinement B type": {
-        "choiceList": ["surface", "radial"],
-        "enabled": True,
-        "group": "Topography refinement",
-        "label": "Type",
-        "value": "surface",
-    },
-    "Refinement B distance": {
-        "enabled": True,
-        "group": "Topography refinement",
-        "label": "Distance",
-        "value": 5000.0,
-    },
-    "ga_group_name": {"enabled": True, "label": "Name:", "value": "Inversion Mesh"},
-    "run_command": "geoapps.octree_creation.application",
-    "run_command_boolean": {
-        "value": False,
-        "label": "Run python module ",
-        "tooltip": "Warning: launches process to run python model on save",
-        "main": True,
-    },
-    "monitoring_directory": None,
-    "workspace_geoh5": None,
-    "conda_environment": "geoapps",
-    "conda_environment_boolean": False,
-}
-
 default_ui_json = {
     "forward_only": False,
     "topography_object": {
         "main": True,
         "group": "Topography",
-        "label": "Object",
+        "label": "Topography",
         "meshType": [
             "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
             "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
@@ -205,14 +27,15 @@ default_ui_json = {
         "value": None,
     },
     "topography": {
-        "association": "Vertex",
+        "association": ["Vertex", "Cell"],
         "dataType": "Float",
         "group": "Topography",
         "main": True,
         "optional": True,
         "enabled": False,
         "isValue": False,
-        "label": "Elevation",
+        "label": "Elevation adjustment",
+        "tooltip": "Adjust elevation given from topography object",
         "parent": "topography_object",
         "property": None,
         "value": 0.0,
@@ -231,35 +54,19 @@ default_ui_json = {
         ],
         "value": None,
     },
-    "starting_model_object": {
-        "group": "Starting Model",
-        "main": True,
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
-            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
-            "{4ea87376-3ece-438b-bf12-3479733ded46}",
-        ],
-        "optional": True,
-        "enabled": False,
-        "label": "Object",
-        "value": None,
-    },
     "starting_model": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Starting Model",
+        "group": "Mesh and Models",
         "main": True,
         "isValue": True,
-        "parent": "starting_model_object",
-        "label": "Value",
+        "parent": "mesh",
+        "label": "Initial Density (g/cc)",
         "property": None,
         "value": 1e-4,
     },
     "tile_spatial": {
-        "group": "Receivers location options",
+        "group": "Compute",
         "label": "Number of tiles",
         "parent": "data_object",
         "isValue": True,
@@ -270,43 +77,45 @@ default_ui_json = {
     },
     "output_tile_files": False,
     "z_from_topo": {
-        "main": False,
-        "group": "Receivers location options",
+        "group": "Data pre-processing",
         "label": "Take z from topography",
+        "tooltip": "Sets survey elevation to topography before any offsets are applied.",
         "value": False,
     },
+    "receivers_offset_x": {
+        "group": "Data pre-processing",
+        "label": "Receiver X offset (m)",
+        "optional": True,
+        "enabled": False,
+        "value": 0.0,
+        "visible": False,
+    },
+    "receivers_offset_y": {
+        "group": "Data pre-processing",
+        "label": "Receiver Y offset (m)",
+        "optional": True,
+        "enabled": False,
+        "value": 0.0,
+        "visible": False,
+    },
+    "receivers_offset_z": {
+        "group": "Data pre-processing",
+        "label": "Z static offset",
+        "optional": True,
+        "enabled": False,
+        "value": 0.0,
+    },
     "receivers_radar_drape": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "main": False,
-        "group": "Receivers location options",
-        "label": "Drape receivers with radar channel",
+        "group": "Data pre-processing",
+        "label": "Z radar offset",
+        "tooltip": "Apply a non-homogeneous offset to survey object from radar channel.",
         "optional": True,
         "parent": "data_object",
         "value": None,
         "enabled": False,
     },
-    "receivers_offset_x": {
-        "group": "Receivers location options",
-        "main": False,
-        "label": "Receiver X offset (m)",
-        "value": 0.0,
-        "enabled": True,
-    },
-    "receivers_offset_y": {
-        "group": "Receivers location options",
-        "main": False,
-        "label": "Receiver Y offset (m)",
-        "value": 0.0,
-        "enabled": True,
-    },
-    "receivers_offset_z": {
-        "group": "Receivers location options",
-        "main": False,
-        "label": "Receiver Z offset (m)",
-        "value": 0.0,
-        "enabled": True,
-    },
     "gps_receivers_offset": None,
     "ignore_values": {
         "group": "Data pre-processing",
@@ -342,33 +151,16 @@ default_ui_json = {
         "label": "Detrend type",
         "value": "all",
     },
-    "max_chunk_size": {
-        "min": 0,
-        "group": "Data pre-processing",
-        "optional": True,
-        "enabled": True,
-        "label": "Maximum chunk size",
-        "value": 128,
-    },
-    "chunk_by_rows": {
-        "group": "Data pre-processing",
-        "label": "Chunk by rows",
-        "value": True,
-    },
     "mesh": {
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
-        "optional": True,
-        "enabled": False,
-        "dependency": "mesh_from_params",
-        "dependencyType": "disabled",
         "label": "Mesh",
         "meshType": "4EA87376-3ECE-438B-BF12-3479733DED46",
         "value": None,
     },
     "u_cell_size": {
         "min": 0.0,
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -378,7 +170,7 @@ default_ui_json = {
     },
     "v_cell_size": {
         "min": 0.0,
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -388,7 +180,7 @@ default_ui_json = {
     },
     "w_cell_size": {
         "min": 0.0,
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -397,7 +189,7 @@ default_ui_json = {
         "value": 25.0,
     },
     "octree_levels_topo": {
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -406,7 +198,7 @@ default_ui_json = {
         "value": [0, 0, 4, 4],
     },
     "octree_levels_obs": {
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -416,7 +208,7 @@ default_ui_json = {
     },
     "depth_core": {
         "min": 0.0,
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -426,7 +218,7 @@ default_ui_json = {
     },
     "max_distance": {
         "min": 0.0,
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -436,7 +228,7 @@ default_ui_json = {
     },
     "horizontal_padding": {
         "min": 0.0,
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -446,7 +238,7 @@ default_ui_json = {
     },
     "vertical_padding": {
         "min": 0.0,
-        "group": "Mesh",
+        "group": "Mesh and Models",
         "main": True,
         "enabled": True,
         "dependency": "mesh",
@@ -456,28 +248,28 @@ default_ui_json = {
     },
     "window_center_x": {
         "group": "Data window",
-        "enabled": True,
+        "enabled": False,
         "groupOptional": True,
         "label": "Window center easting",
         "value": 0.0,
     },
     "window_center_y": {
         "group": "Data window",
-        "enabled": True,
+        "enabled": False,
         "label": "Window center northing",
         "value": 0.0,
     },
     "window_width": {
         "min": 0.0,
         "group": "Data window",
-        "enabled": True,
+        "enabled": False,
         "label": "Window width",
         "value": 0.0,
     },
     "window_height": {
         "min": 0.0,
         "group": "Data window",
-        "enabled": True,
+        "enabled": False,
         "label": "Window height",
         "value": 0.0,
     },
@@ -485,7 +277,7 @@ default_ui_json = {
         "min": -180,
         "max": 180,
         "group": "Data window",
-        "enabled": True,
+        "enabled": False,
         "label": "Window azimuth",
         "value": 0.0,
     },
@@ -500,38 +292,38 @@ default_ui_json = {
     },
     "sens_wts_threshold": {
         "group": "Update sensitivity weights directive",
-        "label": "Update sensitivity weight threshold",
-        "value": 0.0,
+        "tooltip": "Update sensitivity weight threshold",
+        "label": "Threshold (%)",
+        "value": 30.0,
+        "max": 99.9,
+        "min": 0.0,
+        "precision": 1,
+        "lineEdit": False,
     },
     "every_iteration_bool": {
         "group": "Update sensitivity weights directive",
-        "label": "Update every iteration",
+        "tooltip": "Update weights at every iteration",
+        "label": "Every iteration",
         "value": False,
     },
     "f_min_change": {
         "group": "Update IRLS directive",
         "label": "f min change",
         "value": 1e-4,
-    },
-    "minGNiter": {
-        "group": "Update IRLS directive",
-        "label": "Minimum Gauss-Newton iterations",
-        "value": 1,
+        "min": 1e-6,
     },
     "beta_tol": {
         "group": "Update IRLS directive",
         "label": "Beta tolerance",
         "value": 0.5,
+        "min": 0.0001,
     },
     "prctile": {
         "group": "Update IRLS directive",
-        "label": "percentile",
+        "label": "Percentile",
         "value": 95,
-    },
-    "coolingRate": {
-        "group": "Update IRLS directive",
-        "label": "Beta cooling rate",
-        "value": 1,
+        "max": 100,
+        "min": 5,
     },
     "coolEps_q": {
         "group": "Update IRLS directive",
@@ -557,26 +349,41 @@ default_ui_json = {
         "tooltip": "This chi factor will be used to determine the misfit"
         " threshold after which IRLS iterations begin.",
     },
-    "max_iterations": {
-        "min": 0,
+    "max_global_iterations": {
+        "min": 1,
+        "lineEdit": False,
         "group": "Optimization",
+        "label": "Maximum iterations",
+        "tooltip": "Number of L2 and IRLS iterations combined",
+        "value": 100,
+        "enabled": True,
+    },
+    "max_irls_iterations": {
+        "min": 0,
+        "group": "Update IRLS directive",
         "label": "Maximum number of IRLS iterations",
         "tooltip": "Incomplete Re-weighted Least Squares iterations for non-L2 problems",
         "value": 25,
         "enabled": True,
     },
-    "max_global_iterations": {
-        "min": 0,
+    "coolingRate": {
         "group": "Optimization",
-        "label": "Max iterations",
-        "tooltip": "Number of L2 and IRLS iterations combined",
-        "value": 100,
-        "enabled": True,
+        "label": "Iterations per beta",
+        "value": 1,
+        "min": 1,
+    },
+    "coolingFactor": {
+        "group": "Optimization",
+        "label": "Beta cooling factor",
+        "tooltip": "Each beta cooling step will be calculated by dividing the current beta by this factor.",
+        "value": 2.0,
+        "min": 1.0,
     },
     "max_line_search_iterations": {
         "group": "Optimization",
         "label": "Maximum number of line searches",
         "value": 20,
+        "min": 1,
         "enabled": True,
     },
     "max_cg_iterations": {
@@ -588,18 +395,16 @@ default_ui_json = {
     },
     "initial_beta_ratio": {
         "min": 0.0,
+        "precision": 2,
         "group": "Optimization",
         "optional": True,
         "enabled": True,
-        "dependency": "initial_beta",
-        "dependencyType": "disabled",
         "label": "Initial beta ratio",
         "value": 100.0,
     },
     "initial_beta": {
         "min": 0.0,
         "group": "Optimization",
-        "optional": True,
         "enabled": False,
         "dependency": "initial_beta_ratio",
         "dependencyType": "disabled",
@@ -618,12 +423,14 @@ default_ui_json = {
         "group": "Regularization",
         "label": "Smallness weight",
         "value": 1.0,
+        "tooltip": "Constant ratio compared to other weights. Larger values result in models that remain close to the reference model",
         "enabled": True,
     },
     "alpha_x": {
         "min": 0.0,
         "group": "Regularization",
         "label": "X-smoothness weight",
+        "tooltip": "Larger values relative to other smoothness weights will result in x biased smoothness",
         "value": 1.0,
         "enabled": True,
     },
@@ -631,6 +438,7 @@ default_ui_json = {
         "min": 0.0,
         "group": "Regularization",
         "label": "Y-smoothness weight",
+        "tooltip": "Larger values relative to other smoothness weights will result in y biased smoothness",
         "value": 1.0,
         "enabled": True,
     },
@@ -638,6 +446,7 @@ default_ui_json = {
         "min": 0.0,
         "group": "Regularization",
         "label": "Z-smoothness weight",
+        "tooltip": "Larger values relative to other smoothness weights will result in z biased smoothess",
         "value": 1.0,
         "enabled": True,
     },
@@ -647,6 +456,8 @@ default_ui_json = {
         "group": "Regularization",
         "label": "Smallness norm",
         "value": 0.0,
+        "precision": 2,
+        "lineEdit": False,
         "enabled": True,
     },
     "x_norm": {
@@ -655,6 +466,8 @@ default_ui_json = {
         "group": "Regularization",
         "label": "X-smoothness norm",
         "value": 2.0,
+        "precision": 2,
+        "lineEdit": False,
         "enabled": True,
     },
     "y_norm": {
@@ -663,6 +476,8 @@ default_ui_json = {
         "group": "Regularization",
         "label": "Y-smoothness norm",
         "value": 2.0,
+        "precision": 2,
+        "lineEdit": False,
         "enabled": True,
     },
     "z_norm": {
@@ -671,34 +486,19 @@ default_ui_json = {
         "group": "Regularization",
         "label": "Z-smoothness norm",
         "value": 2.0,
+        "precision": 2,
+        "lineEdit": False,
         "enabled": True,
     },
-    "reference_model_object": {
-        "group": "Regularization",
-        "label": "Reference model object",
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
-            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
-            "{4ea87376-3ece-438b-bf12-3479733ded46}",
-        ],
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
     "reference_model": {
         "association": ["Cell", "Vertex"],
+        "main": True,
         "dataType": "Float",
-        "group": "Regularization",
+        "group": "Mesh and Models",
         "isValue": True,
-        "parent": "reference_model_object",
-        "dependency": "reference_model_object",
-        "dependencyType": "enabled",
-        "label": "Reference model value",
+        "parent": "mesh",
+        "label": "Reference",
         "property": None,
-        "optional": True,
         "value": 0.0,
     },
     "gradient_type": {
@@ -707,61 +507,31 @@ default_ui_json = {
         "label": "Gradient type",
         "value": "total",
     },
-    "lower_bound_object": {
-        "group": "Regularization",
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
-            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
-            "{4ea87376-3ece-438b-bf12-3479733ded46}",
-        ],
-        "label": "Lower bound object",
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
     "lower_bound": {
         "association": ["Cell", "Vertex"],
+        "main": True,
         "dataType": "Float",
-        "group": "Regularization",
-        "isValue": False,
-        "parent": "lower_bound_object",
-        "dependency": "lower_bound_object",
-        "dependencyType": "enabled",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
         "label": "Lower bound",
         "property": None,
         "optional": True,
-        "value": 0.0,
-    },
-    "upper_bound_object": {
-        "group": "Regularization",
-        "label": "Upper bound object",
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
-            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
-            "{4ea87376-3ece-438b-bf12-3479733ded46}",
-        ],
-        "optional": True,
+        "value": -10.0,
         "enabled": False,
-        "value": None,
     },
     "upper_bound": {
         "association": ["Cell", "Vertex"],
+        "main": True,
         "dataType": "Float",
-        "group": "Regularization",
-        "isValue": False,
-        "parent": "upper_bound_object",
-        "parent": "upper_bound_object",
-        "dependency": "lower_bound_object",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
         "label": "Upper bound",
         "property": None,
         "optional": True,
-        "value": 0.0,
+        "value": 10.0,
+        "enabled": False,
     },
     "parallelized": {
         "group": "Compute",
@@ -778,6 +548,25 @@ default_ui_json = {
         "label": "Number of cpu",
         "value": 1,
     },
+    "store_sensitivities": {
+        "choiceList": ["disk", "ram"],
+        "group": "Compute",
+        "label": "Storage device",
+        "value": "disk",
+    },
+    "max_chunk_size": {
+        "min": 0,
+        "group": "Compute",
+        "optional": True,
+        "enabled": True,
+        "label": "Maximum chunk size",
+        "value": 128,
+    },
+    "chunk_by_rows": {
+        "group": "Compute",
+        "label": "Chunk by rows",
+        "value": True,
+    },
     "max_ram": None,
     "monitoring_directory": None,
     "workspace_geoh5": None,
@@ -797,8 +586,15 @@ default_ui_json = {
 
 validations = {
     "topography_object": {
-        "required": True,
         "types": [str, UUID, Surface, Points, Grid2D, Curve],
     },
-    "out_group": {"required": True, "types": [str, ContainerGroup]},
+    "alpha_s": {"types": [int, float]},
+    "alpha_x": {"types": [int, float]},
+    "alpha_y": {"types": [int, float]},
+    "alpha_z": {"types": [int, float]},
+    "norm_s": {"types": [int, float]},
+    "norm_x": {"types": [int, float]},
+    "norm_y": {"types": [int, float]},
+    "norm_z": {"types": [int, float]},
+    "distributed_workers": {"types": [tuple, type(None)]},
 }
