@@ -5,5 +5,10 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
-from .direct_current.params import DirectCurrentParams
-from .induced_polarization.params import InducedPolarizationParams
+from __future__ import annotations
+
+from geoapps.inversion.electricals.induced_polarization.three_dimensions.params import (
+    InducedPolarization3DParams,
+)
+
+from .direct_current.three_dimensions import DirectCurrent3DParams
