@@ -5,43 +5,55 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 import json
 import multiprocessing
 import os
 import os.path as path
 import uuid
+import warnings
+from time import time
 
-import ipywidgets as widgets
 import numpy as np
-from geoh5py.objects import (
-    BlockModel,
-    CurrentElectrode,
-    Curve,
-    Octree,
-    Points,
-    PotentialElectrode,
-    Surface,
-)
+from geoh5py.data import Data
+from geoh5py.objects import CurrentElectrode, PotentialElectrode
 from geoh5py.ui_json import InputFile
 from geoh5py.workspace import Workspace
-from ipywidgets.widgets import (
-    Button,
-    Checkbox,
-    Dropdown,
-    FloatText,
-    HBox,
-    IntText,
-    Label,
-    Layout,
-    VBox,
-    Widget,
-)
 
-from geoapps import PlotSelection2D
+from geoapps.base.application import BaseApplication
+from geoapps.base.plot import PlotSelection2D
 from geoapps.base.selection import ObjectDataSelection, TopographyOptions
-from geoapps.inversion.electricals import DirectCurrentParams, InducedPolarizationParams
-from geoapps.inversion.electricals.direct_current.constants import app_initializer
-from geoapps.utils.utils import find_value, string_2_list
+from geoapps.inversion.electricals.direct_current.three_dimensions.constants import (
+    app_initializer,
+)
+from geoapps.inversion.electricals.direct_current.three_dimensions.params import (
+    DirectCurrent3DParams,
+)
+from geoapps.inversion.electricals.induced_polarization.three_dimensions.params import (
+    InducedPolarization3DParams,
+)
+from geoapps.inversion.potential_fields.application import (
+    MeshOctreeOptions,
+    ModelOptions,
+)
+from geoapps.utils import warn_module_not_found
+from geoapps.utils.list import find_value
+
+with warn_module_not_found():
+    import ipywidgets as widgets
+    from ipywidgets.widgets import (
+        Button,
+        Checkbox,
+        Dropdown,
+        FloatText,
+        HBox,
+        IntText,
+        Label,
+        Layout,
+        VBox,
+        Widget,
+    )
 
 
 def inversion_defaults():
@@ -77,16 +89,16 @@ class InversionApp(PlotSelection2D):
     Application for the inversion of potential field data using SimPEG
     """
 
-    _param_class = DirectCurrentParams
+    _param_class = DirectCurrent3DParams
     _select_multiple = True
     _add_groups = False
     _sensor = None
     _lines = None
     _topography = None
+    _run_params = None
     _object_types = (PotentialElectrode,)
     _exclusion_types = (CurrentElectrode,)
     inversion_parameters = None
-    defaults = {}
 
     def __init__(self, ui_json=None, **kwargs):
         if "plot_result" in kwargs:
@@ -124,17 +136,35 @@ class InversionApp(PlotSelection2D):
         self._chi_factor = FloatText(
             value=1, description="Target misfit", disabled=False
         )
+        self._mesh_octree = MeshOctreeOptions(**self.defaults)
         self._lower_bound_group = ModelOptions(
-            "lower_bound", add_xyz=False, **self.defaults
+            "lower_bound",
+            add_xyz=False,
+            objects=self._mesh_octree.mesh,
+            **self.defaults,
         )
         self._upper_bound_group = ModelOptions(
-            "upper_bound", add_xyz=False, **self.defaults
+            "upper_bound",
+            add_xyz=False,
+            objects=self._mesh_octree.mesh,
+            **self.defaults,
         )
         self._ignore_values = widgets.Text(
             description="Value (i.e. '<0' for no negatives)",
         )
-        self._max_iterations = IntText(value=10, description="Max beta Iterations")
+        self._store_sensitivities = Dropdown(
+            options=["ram", "disk"], description="Storage device:", value="disk"
+        )
+        self._max_global_iterations = IntText(
+            value=10, description="Max beta Iterations"
+        )
+        self._max_irls_iterations = IntText(value=10, description="Max IRLS iterations")
+        self._coolingRate = IntText(value=1, description="Iterations per beta")
+        self._coolingFactor = FloatText(value=2, description="Beta cooling factor")
         self._max_cg_iterations = IntText(value=30, description="Max CG Iterations")
+        self._sens_wts_threshold = FloatText(
+            value=80, description="Threshold sensitivity weights", max=100, min=0
+        )
         self._tol_cg = FloatText(value=1e-3, description="CG Tolerance")
         self._n_cpu = IntText(
             value=int(multiprocessing.cpu_count() / 2), description="Max CPUs"
@@ -146,25 +176,40 @@ class InversionApp(PlotSelection2D):
         self._initial_beta_panel = HBox([self._initial_beta_ratio])
         self._optimization = VBox(
             [
-                self._max_iterations,
+                self._max_global_iterations,
+                self._max_irls_iterations,
+                self._coolingRate,
+                self._coolingFactor,
                 self._chi_factor,
                 self._initial_beta_panel,
                 self._max_cg_iterations,
                 self._tol_cg,
                 self._n_cpu,
+                self._store_sensitivities,
+                self._sens_wts_threshold,
                 self._tile_spatial,
             ]
         )
         self._starting_model_group = ModelOptions(
-            "starting_model", add_xyz=False, **self.defaults
+            "starting_model",
+            add_xyz=False,
+            objects=self._mesh_octree.mesh,
+            **self.defaults,
         )
         self._starting_model_group.options.options = ["Constant", "Model"]
+        self._starting_model_group.options.value = "Constant"
         self._conductivity_model_group = ModelOptions(
-            "conductivity_model", add_xyz=False, **self.defaults
+            "conductivity_model",
+            add_xyz=False,
+            objects=self._mesh_octree.mesh,
+            **self.defaults,
         )
         self._conductivity_model_group.options.options = ["Model"]
         self._reference_model_group = ModelOptions(
-            "reference_model", add_xyz=False, **self.defaults
+            "reference_model",
+            add_xyz=False,
+            objects=self._mesh_octree.mesh,
+            **self.defaults,
         )
         self._reference_model_group.options.observe(self.update_ref)
         self._topography_group = TopographyOptions(add_xyz=False, **self.defaults)
@@ -174,7 +219,9 @@ class InversionApp(PlotSelection2D):
             object_types=self._object_types,
             exclusion_types=self._exclusion_types,
             add_xyz=False,
-            **self.defaults,
+            receivers_offset_z=self.defaults["receivers_offset_z"],
+            z_from_topo=self.defaults["z_from_topo"],
+            receivers_radar_drape=self.defaults["receivers_radar_drape"],
         )
         self._alpha_s = widgets.FloatText(
             min=0,
@@ -245,7 +292,6 @@ class InversionApp(PlotSelection2D):
                 ),
             ]
         )
-        self._mesh_octree = MeshOctreeOptions(**self.defaults)
         self.inversion_options = {
             "conductivity model": VBox(
                 [
@@ -269,8 +315,8 @@ class InversionApp(PlotSelection2D):
             "optimization": self._optimization,
         }
         self.option_choices = widgets.Dropdown(
-            options=list(self.inversion_options.keys())[1:],
-            value=list(self.inversion_options.keys())[1],
+            options=list(self.inversion_options)[1:],
+            value=list(self.inversion_options)[1],
             disabled=False,
         )
         self.option_choices.observe(self.inversion_option_change, names="value")
@@ -282,11 +328,14 @@ class InversionApp(PlotSelection2D):
         self.data_channel_choices.observe(
             self.data_channel_choices_observer, names="value"
         )
+        self.plotting = None
+        self._starting_channel = None
+        self._mesh = self._mesh_octree.mesh
+        self._detrend_type = None
+        self._detrend_order = None
+        self._initial_beta_options = None
         super().__init__(**self.defaults)
 
-        for item in ["window_width", "window_height", "resolution"]:
-            getattr(self, item).observe(self.update_octree_param, names="value")
-
         self.write.on_click(self.write_trigger)
 
     @property
@@ -313,6 +362,14 @@ class InversionApp(PlotSelection2D):
     # def initial_beta(self):
     #     return self._initial_beta
 
+    @property
+    def coolingRate(self):
+        return self._coolingRate
+
+    @property
+    def coolingFactor(self):
+        return self._coolingFactor
+
     @property
     def initial_beta(self):
         return self._initial_beta_ratio
@@ -338,8 +395,12 @@ class InversionApp(PlotSelection2D):
         return self._ignore_values
 
     @property
-    def max_iterations(self):
-        return self._max_iterations
+    def max_global_iterations(self):
+        return self._max_global_iterations
+
+    @property
+    def max_irls_iterations(self):
+        return self._max_irls_iterations
 
     @property
     def max_cg_iterations(self):
@@ -395,6 +456,10 @@ class InversionApp(PlotSelection2D):
     def ga_group_name(self):
         return self._ga_group_name
 
+    @property
+    def store_sensitivities(self):
+        return self._store_sensitivities
+
     @property
     def reference_model(self):
         if self._reference_model_group.options.value == "Model":
@@ -414,10 +479,6 @@ class InversionApp(PlotSelection2D):
         else:
             self._reference_model_group.data.value = value
 
-    @property
-    def reference_model_object(self):
-        return self._reference_model_group.objects
-
     @property
     def starting_model(self):
         if self._starting_model_group.options.value == "Model":
@@ -435,10 +496,6 @@ class InversionApp(PlotSelection2D):
         else:
             self._starting_model_group.data.value = value
 
-    @property
-    def starting_model_object(self):
-        return self._starting_model_group.objects
-
     @property
     def conductivity_model(self):
         if self._conductivity_model_group.options.value == "Model":
@@ -456,10 +513,6 @@ class InversionApp(PlotSelection2D):
         else:
             self._conductivity_model_group.data.value = value
 
-    @property
-    def conductivity_model_object(self):
-        return self._conductivity_model_group.objects
-
     @property
     def lower_bound(self):
         if self._lower_bound_group.options.value == "Model":
@@ -479,10 +532,6 @@ class InversionApp(PlotSelection2D):
         else:
             self._lower_bound_group.data.value = value
 
-    @property
-    def lower_bound_object(self):
-        return self._lower_bound_group.objects
-
     @property
     def upper_bound(self):
         if self._upper_bound_group.options.value == "Model":
@@ -503,8 +552,8 @@ class InversionApp(PlotSelection2D):
             self._upper_bound_group.data.value = value
 
     @property
-    def upper_bound_object(self):
-        return self._upper_bound_group.objects
+    def sens_wts_threshold(self):
+        return self._sens_wts_threshold
 
     @property
     def tol_cg(self):
@@ -609,14 +658,6 @@ class InversionApp(PlotSelection2D):
     def receivers_radar_drape(self):
         return self.sensor.data
 
-    @property
-    def receivers_offset_x(self):
-        return self.sensor.receivers_offset_x
-
-    @property
-    def receivers_offset_y(self):
-        return self.sensor.receivers_offset_y
-
     @property
     def receivers_offset_z(self):
         return self.sensor.receivers_offset_z
@@ -683,6 +724,7 @@ class InversionApp(PlotSelection2D):
         self._starting_model_group.workspace = workspace
         self._conductivity_model_group.workspace = workspace
         self._mesh_octree.workspace = workspace
+        self.plotting_data = None
 
     @property
     def write(self):
@@ -706,7 +748,11 @@ class InversionApp(PlotSelection2D):
 
     def trigger_click(self, _):
         """"""
-        self.run(self.params)
+        if self._run_params is None:
+            warnings.warn("Input file must be written before running.")
+            return
+
+        self.run(self._run_params)
         self.trigger.button_style = ""
 
     def inversion_type_observer(self, _):
@@ -715,20 +761,20 @@ class InversionApp(PlotSelection2D):
         """
         params = self.params.to_dict(ui_json_format=False)
         if self.inversion_type.value == "direct current" and not isinstance(
-            self.params, DirectCurrentParams
+            self.params, DirectCurrent3DParams
         ):
-            self._param_class = DirectCurrentParams
+            self._param_class = DirectCurrent3DParams
             params["inversion_type"] = "direct current"
             params["out_group"] = "DCInversion"
-            self.option_choices.options = list(self.inversion_options.keys())[1:]
+            self.option_choices.options = list(self.inversion_options)[1:]
 
         elif self.inversion_type.value == "induced polarization" and not isinstance(
-            self.params, InducedPolarizationParams
+            self.params, InducedPolarization3DParams
         ):
-            self._param_class = InducedPolarizationParams
+            self._param_class = InducedPolarization3DParams
             params["inversion_type"] = "induced polarization"
             params["out_group"] = "ChargeabilityInversion"
-            self.option_choices.options = list(self.inversion_options.keys())
+            self.option_choices.options = list(self.inversion_options)
 
         self.params = self._param_class(
             # validator_opts={"ignore_requirements": True}
@@ -743,6 +789,7 @@ class InversionApp(PlotSelection2D):
 
         flag = self.inversion_type.value
         self._reference_model_group.units = inversion_defaults()["units"][flag]
+        self._reference_model_group.options.value = "Constant"
         self._reference_model_group.constant.value = inversion_defaults()[
             "reference_value"
         ][flag]
@@ -750,6 +797,7 @@ class InversionApp(PlotSelection2D):
             "Reference " + inversion_defaults()["property"][flag]
         )
         self._starting_model_group.units = inversion_defaults()["units"][flag]
+        self._starting_model_group.options.value = "Constant"
         self._starting_model_group.constant.value = inversion_defaults()[
             "starting_value"
         ][flag]
@@ -758,9 +806,9 @@ class InversionApp(PlotSelection2D):
         )
         data_channel_options = {}
         self.data_channel_choices.options = data_type_list
+        obj, _ = self.get_selected_entities()
 
-        if self.workspace.get_entity(self.objects.value):
-            obj, _ = self.get_selected_entities()
+        if obj is not None:
             options = [
                 [name, obj.get_data(name)[0].uid]
                 for name in sorted(obj.get_data_list())
@@ -775,6 +823,10 @@ class InversionApp(PlotSelection2D):
                 channel = caller["owner"]
                 data_widget = getattr(self, f"{channel.header}_group")
                 entity = self.workspace.get_entity(self.objects.value)[0]
+
+                if entity is None:
+                    return
+
                 if channel.value is None or channel.value not in [
                     child.uid for child in entity.children
                 ]:
@@ -876,8 +928,6 @@ class InversionApp(PlotSelection2D):
                 ["", None]
             ] + options
 
-            # data_channel_options[key].children[1].value = find_value(options, [key])
-
         self.data_channel_choices.value = inversion_defaults()["component"][
             self.inversion_type.value
         ]
@@ -887,6 +937,7 @@ class InversionApp(PlotSelection2D):
             data_channel_options[self.data_channel_choices.value],
         ]
         self.write.button_style = "warning"
+        self._run_params = None
         self.trigger.button_style = "danger"
 
         if self.inversion_type.value == "direct current":
@@ -899,31 +950,34 @@ class InversionApp(PlotSelection2D):
     def object_observer(self, _):
         """ """
         self.resolution.indices = None
-        if self.workspace.get_entity(self.objects.value):
-            self.update_data_list(None)
-            self.sensor.update_data_list(None)
-            # self.lines.update_data_list(None)
-            # self.lines.update_line_list(None)
-            self.inversion_type_observer(None)
-            self.write.button_style = "warning"
-            self.trigger.button_style = "danger"
+        obj = self.workspace.get_entity(self.objects.value)[0]
+        if obj is None:
+            return
+
+        self.update_data_list(None)
+        self.sensor.update_data_list(None)
+        self.inversion_type_observer(None)
+        self.write.button_style = "warning"
+        self._run_params = None
+        self.trigger.button_style = "danger"
 
     def data_channel_choices_observer(self, _):
         if hasattr(
             self.data_channel_choices, "data_channel_options"
         ) and self.data_channel_choices.value in (
-            self.data_channel_choices.data_channel_options.keys()
+            self.data_channel_choices.data_channel_options
         ):
             data_widget = self.data_channel_choices.data_channel_options[
                 self.data_channel_choices.value
             ]
             self.data_channel_panel.children = [self.data_channel_choices, data_widget]
-
+            obj, data_list = self.get_selected_entities()
             if (
-                self.workspace.get_entity(self.objects.value)
+                obj is not None
+                and data_list is not None
                 and data_widget.children[1].value is None
             ):
-                _, data_list = self.get_selected_entities()
+
                 options = [[data.name, data.uid] for data in data_list]
                 data_widget.children[1].value = find_value(
                     options, [self.data_channel_choices.value]
@@ -934,27 +988,13 @@ class InversionApp(PlotSelection2D):
             self.refresh.value = True
 
         self.write.button_style = "warning"
-        self.trigger.button_style = "danger"
-
-    def update_octree_param(self, _):
-        dl = self.resolution.value
-        self._mesh_octree.u_cell_size.value = f"{dl/2:.0f}"
-        self._mesh_octree.v_cell_size.value = f"{dl / 2:.0f}"
-        self._mesh_octree.w_cell_size.value = f"{dl / 2:.0f}"
-        self._mesh_octree.depth_core.value = np.ceil(
-            np.min([self.window_width.value, self.window_height.value]) / 2.0
-        )
-        self._mesh_octree.horizontal_padding.value = (
-            np.max([self.window_width.value, self.window_width.value]) / 2
-        )
-        self.resolution.indices = None
-        self.write.button_style = "warning"
+        self._run_params = None
         self.trigger.button_style = "danger"
 
     def write_trigger(self, _):
 
         if (
-            self.inversion_type.value == "induced polarization"
+            "induced polarization" in self.inversion_type.value
             and self._conductivity_model_group.data.value is None
         ):
             print(
@@ -962,93 +1002,130 @@ class InversionApp(PlotSelection2D):
             )
             return
 
-        new_workspace = Workspace(
-            path.join(
-                self.export_directory.selected_path,
-                self._ga_group_name.value + ".geoh5",
-            )
-        )
-
-        for elem in [
-            self,
-            self._mesh_octree,
-            self._topography_group,
-            self._starting_model_group,
-            self._conductivity_model_group,
-            self._reference_model_group,
-            self._lower_bound_group,
-            self._upper_bound_group,
-        ]:
-            obj, data = elem.get_selected_entities()
-            if obj is not None:
-                new_obj = new_workspace.get_entity(obj.uid)[0]
-                if new_obj is None:
-                    new_obj = obj.copy(parent=new_workspace, copy_children=False)
-                for d in data:
-                    if new_workspace.get_entity(d.uid)[0] is None:
-                        d.copy(parent=new_obj)
-
-        new_obj = new_workspace.get_entity(self.objects.value)[0]
-        for key in self.data_channel_choices.options:
-            widget = getattr(self, f"{key}_uncertainty_channel")
-            if widget.value is not None:
-                setattr(self.params, f"{key}_uncertainty", str(widget.value))
-                if new_workspace.get_entity(widget.value)[0] is None:
-                    self.workspace.get_entity(widget.value)[0].copy(
-                        parent=new_obj, copy_children=False
-                    )
-            else:
-                widget = getattr(self, f"{key}_uncertainty_floor")
-                setattr(self.params, f"{key}_uncertainty", widget.value)
-
-            if getattr(self, f"{key}_channel_bool").value:
-                self.workspace.get_entity(getattr(self, f"{key}_channel").value)[
-                    0
-                ].copy(parent=new_obj)
-
-        if self.receivers_radar_drape.value is not None:
-            self.workspace.get_entity(self.receivers_radar_drape.value)[0].copy(
-                parent=new_obj
-            )
-
-        self.params.geoh5 = new_workspace
-
+        # Widgets values populate params dictionary
+        param_dict = {}
         for key in self.__dict__:
-            if "resolution" in key:
+            try:
+                if isinstance(getattr(self, key), Widget) and hasattr(self.params, key):
+                    value = getattr(self, key).value
+                    if key[0] == "_":
+                        key = key[1:]
+
+                    if (
+                        isinstance(value, uuid.UUID)
+                        and self.workspace.get_entity(value)[0] is not None
+                    ):
+                        value = self.workspace.get_entity(value)[0]
+
+                    param_dict[key] = value
+
+            except AttributeError:
                 continue
 
-            attr = getattr(self, key)
-            if isinstance(attr, Widget) and hasattr(attr, "value"):
-                value = attr.value
-                if isinstance(value, uuid.UUID):
-                    value = new_workspace.get_entity(value)[0]
-                setattr(self.params, key, value)
-            else:
-                sub_keys = []
-                if isinstance(attr, (ModelOptions, TopographyOptions)):
-                    sub_keys = [attr.identifier, attr.identifier + "_object"]
-                    attr = self
-                elif isinstance(attr, (MeshOctreeOptions, SensorOptions)):
-                    sub_keys = attr.params_keys
-                for sub_key in sub_keys:
-                    value = getattr(attr, sub_key)
-                    if isinstance(value, Widget) and hasattr(value, "value"):
-                        value = value.value
+        # Create a new workapce and copy objects into it
+        temp_geoh5 = f"{self.ga_group_name.value}_{time():.0f}.geoh5"
+        ws, self.live_link.value = BaseApplication.get_output_workspace(
+            self.live_link.value, self.export_directory.selected_path, temp_geoh5
+        )
+        with ws as new_workspace:
+
+            param_dict["geoh5"] = new_workspace
+
+            for elem in [
+                self,
+                self._mesh_octree,
+                self._topography_group,
+                self._starting_model_group,
+                self._conductivity_model_group,
+                self._reference_model_group,
+                self._lower_bound_group,
+                self._upper_bound_group,
+            ]:
+                obj, data = elem.get_selected_entities()
+                if obj is not None:
+                    new_obj = new_workspace.get_entity(obj.uid)[0]
+                    if new_obj is None:
+                        new_obj = obj.copy(parent=new_workspace, copy_children=False)
+                    for d in data:
+                        if (
+                            isinstance(d, Data)
+                            and new_workspace.get_entity(d.uid)[0] is None
+                        ):
+                            d.copy(parent=new_obj)
+
+            new_obj = new_workspace.get_entity(self.objects.value)[0]
+
+            for key in self.data_channel_choices.options:
+                if not self.forward_only.value:
+                    widget = getattr(self, f"{key}_uncertainty_channel")
+                    if widget.value is not None:
+                        param_dict[f"{key}_uncertainty"] = str(widget.value)
+                        if new_workspace.get_entity(widget.value)[0] is None:
+                            self.workspace.get_entity(widget.value)[0].copy(
+                                parent=new_obj, copy_children=False
+                            )
+                    else:
+                        widget = getattr(self, f"{key}_uncertainty_floor")
+                        param_dict[f"{key}_uncertainty"] = widget.value
+
+                if getattr(self, f"{key}_channel_bool").value:
+                    if not self.forward_only.value:
+                        self.workspace.get_entity(
+                            getattr(self, f"{key}_channel").value
+                        )[0].copy(parent=new_obj)
+                    else:
+                        param_dict[f"{key}_channel_bool"] = True
+
+            if self.receivers_radar_drape.value is not None:
+                self.workspace.get_entity(self.receivers_radar_drape.value)[0].copy(
+                    parent=new_obj
+                )
+
+            for key in self.__dict__:
+                if "resolution" in key:
+                    continue
+
+                attr = getattr(self, key)
+                if isinstance(attr, Widget) and hasattr(attr, "value"):
+                    value = attr.value
                     if isinstance(value, uuid.UUID):
                         value = new_workspace.get_entity(value)[0]
-                    setattr(self.params, sub_key, value)
+                    if hasattr(self.params, key):
+                        param_dict[key.lstrip("_")] = value
+                else:
+                    sub_keys = []
+                    if isinstance(attr, TopographyOptions):
+                        sub_keys = [attr.identifier, attr.identifier + "_object"]
+                        attr = self
+                    elif isinstance(attr, ModelOptions):
+                        sub_keys = [attr.identifier]
+                        attr = self
+                    elif isinstance(attr, (MeshOctreeOptions, SensorOptions)):
+                        sub_keys = attr.params_keys
+                    for sub_key in sub_keys:
+                        value = getattr(attr, sub_key)
+                        if isinstance(value, Widget) and hasattr(value, "value"):
+                            value = value.value
+                        if isinstance(value, uuid.UUID):
+                            value = new_workspace.get_entity(value)[0]
+                        if hasattr(self.params, sub_key):
+                            param_dict[sub_key.lstrip("_")] = value
+
+            # Create new params object and write
+            param_dict["resolution"] = None  # No downsampling for dcip
+            self._run_params = self.params.__class__(**param_dict)
+            self._run_params.write_input_file(
+                name=temp_geoh5.replace(".geoh5", ".ui.json"),
+                path=self.export_directory.selected_path,
+            )
 
-        self.params.write_input_file(
-            name=self._ga_group_name.value + ".ui.json",
-            path=self.export_directory.selected_path,
-        )
         self.write.button_style = ""
         self.trigger.button_style = "success"
 
     @staticmethod
     def run(params):
 
-        if not isinstance(params, (DirectCurrentParams, InducedPolarizationParams)):
+        if not isinstance(params, (DirectCurrent3DParams, InducedPolarization3DParams)):
             raise ValueError(
                 "Parameter 'inversion_type' must be one of "
                 "'direct current' or 'induced polarization'"
@@ -1064,24 +1141,28 @@ class InversionApp(PlotSelection2D):
         """
         Change the target h5file
         """
-        if not self.file_browser._select.disabled:
+        if not self.file_browser._select.disabled:  # pylint: disable=protected-access
             _, extension = path.splitext(self.file_browser.selected)
 
+            if isinstance(self.geoh5, Workspace):
+                self.geoh5.close()
+
             if extension == ".json" and getattr(self, "_param_class", None) is not None:
 
                 # Read the inversion type first...
-                with open(self.file_browser.selected) as f:
+                with open(self.file_browser.selected, encoding="utf8") as f:
                     data = json.load(f)
 
                 if data["inversion_type"] == "direct current":
-                    self._param_class = DirectCurrentParams
+                    self._param_class = DirectCurrent3DParams
 
                 elif data["inversion_type"] == "induced polarization":
-                    self._param_class = InducedPolarizationParams
+                    self._param_class = InducedPolarization3DParams
 
                 self.params = getattr(self, "_param_class")(
                     InputFile.read_ui_json(self.file_browser.selected)
                 )
+                self.params.geoh5.open(mode="r")
                 self.refresh.value = False
                 self.__populate__(**self.params.to_dict(ui_json_format=False))
                 self.refresh.value = True
@@ -1098,8 +1179,6 @@ class SensorOptions(ObjectDataSelection):
     _options = None
     defaults = {}
     params_keys = [
-        "receivers_offset_x",
-        "receivers_offset_y",
         "receivers_offset_z",
         "z_from_topo",
         "receivers_radar_drape",
@@ -1107,12 +1186,13 @@ class SensorOptions(ObjectDataSelection):
 
     def __init__(self, **kwargs):
         self.defaults.update(**kwargs)
-        self._receivers_offset_x = FloatText(description="dx (+East)", value=0.0)
-        self._receivers_offset_y = FloatText(description="dy (+North)", value=0.0)
         self._receivers_offset_z = FloatText(description="dz (+ve up)", value=0.0)
-        self._z_from_topo = Checkbox(description="Set Z from topo + offsets")
+        self._z_from_topo = Checkbox(
+            description="Set Z from topo + offsets", value=True
+        )
         self.data.description = "Radar (Optional):"
         self._receivers_radar_drape = self.data
+        self._offset = None
         super().__init__(**self.defaults)
 
     @property
@@ -1122,8 +1202,6 @@ class SensorOptions(ObjectDataSelection):
                 [
                     self.z_from_topo,
                     Label("Offsets"),
-                    self._receivers_offset_x,
-                    self._receivers_offset_y,
                     self._receivers_offset_z,
                     # self._receivers_radar_drape,
                 ]
@@ -1131,21 +1209,13 @@ class SensorOptions(ObjectDataSelection):
 
         return self._main
 
-    @property
-    def offset(self):
-        return self._offset
-
     @property
     def receivers_radar_drape(self):
         return self._receivers_radar_drape
 
     @property
-    def receivers_offset_x(self):
-        return self._receivers_offset_x
-
-    @property
-    def receivers_offset_y(self):
-        return self._receivers_offset_y
+    def offset(self):
+        return self._offset
 
     @property
     def receivers_offset_z(self):
@@ -1154,216 +1224,3 @@ class SensorOptions(ObjectDataSelection):
     @property
     def z_from_topo(self):
         return self._z_from_topo
-
-
-class MeshOctreeOptions(ObjectDataSelection):
-    """
-    Widget used for the creation of an octree meshes
-    """
-
-    params_keys = [
-        "mesh",
-        "u_cell_size",
-        "v_cell_size",
-        "w_cell_size",
-        "octree_levels_topo",
-        "octree_levels_obs",
-        "depth_core",
-        "horizontal_padding",
-        "vertical_padding",
-        "max_distance",
-    ]
-
-    def __init__(self, **kwargs):
-        self._mesh = self.objects
-        self._u_cell_size = widgets.FloatText(
-            value=25.0,
-            description="",
-        )
-        self._v_cell_size = widgets.FloatText(
-            value=25.0,
-            description="",
-        )
-        self._w_cell_size = widgets.FloatText(
-            value=25.0,
-            description="",
-        )
-        self._octree_levels_topo = widgets.Text(
-            value="0, 0, 0, 2",
-            description="# Cells below topography",
-        )
-        self._octree_levels_obs = widgets.Text(
-            value="5, 5, 5, 5",
-            description="# Cells below sensors",
-        )
-        self._depth_core = FloatText(
-            value=500,
-            description="Minimum depth (m)",
-        )
-        self._horizontal_padding = widgets.FloatText(
-            value=1000.0,
-            description="Horizontal padding (m)",
-        )
-        self._vertical_padding = widgets.FloatText(
-            value=1000.0,
-            description="Vertical padding (m)",
-        )
-        self._max_distance = FloatText(
-            value=1000,
-            description="Maximum distance (m)",
-        )
-        self._parameters = widgets.VBox(
-            [
-                Label("Core cell size (u, v, z)"),
-                self._u_cell_size,
-                self._v_cell_size,
-                self._w_cell_size,
-                Label("Refinement Layers"),
-                self._octree_levels_topo,
-                self._octree_levels_obs,
-                self._max_distance,
-                Label("Dimensions"),
-                self._horizontal_padding,
-                self._vertical_padding,
-                self._depth_core,
-            ]
-        )
-        self._main = VBox([self.objects, self._parameters])
-        self._objects.observe(self.mesh_selection, names="value")
-
-        super().__init__(**kwargs)
-
-    @property
-    def main(self):
-        return self._main
-
-    @property
-    def mesh(self):
-        return self._mesh
-
-    @property
-    def u_cell_size(self):
-        return self._u_cell_size
-
-    @property
-    def v_cell_size(self):
-        return self._v_cell_size
-
-    @property
-    def w_cell_size(self):
-        return self._w_cell_size
-
-    @property
-    def depth_core(self):
-        return self._depth_core
-
-    @property
-    def max_distance(self):
-        return self._max_distance
-
-    @property
-    def octree_levels_obs(self):
-        return self._octree_levels_obs
-
-    @octree_levels_obs.getter
-    def octree_levels_obs(self):
-        return string_2_list(self._octree_levels_obs.value)
-
-    @property
-    def octree_levels_topo(self):
-        return self._octree_levels_topo
-
-    @octree_levels_topo.getter
-    def octree_levels_topo(self):
-        return string_2_list(self._octree_levels_topo.value)
-
-    @property
-    def horizontal_padding(self):
-        return self._horizontal_padding
-
-    @property
-    def vertical_padding(self):
-        return self._vertical_padding
-
-    @property
-    def main(self):
-        return self._main
-
-    def mesh_selection(self, _):
-        if self._mesh.value is None:
-            self._main.children = [
-                self.objects,
-                self._parameters,
-            ]
-        else:
-            self._main.children = [self.objects]
-
-
-class ModelOptions(ObjectDataSelection):
-    """
-    Widgets for the selection of model options
-    """
-
-    def __init__(self, identifier: str = None, **kwargs):
-        self._units = "Units"
-        self._identifier = identifier
-        self._object_types = (BlockModel, Octree, Surface, Curve, Points)
-        self._options = widgets.RadioButtons(
-            options=["Model", "Constant", "None"],
-            value="Constant",
-            disabled=False,
-        )
-        self._options.observe(self.update_panel, names="value")
-        self.objects.description = "Object"
-
-        self.data.description = "Values"
-        self._constant = FloatText(description=self.units)
-        self._description = Label()
-
-        super().__init__(**kwargs)
-
-        self.objects.observe(self.objects_setter, names="value")
-        self.selection_widget = self.main
-        self._main = widgets.VBox(
-            [self._description, widgets.VBox([self._options, self._constant])]
-        )
-
-    def update_panel(self, _):
-
-        if self._options.value == "Model":
-            self._main.children[1].children = [self._options, self.selection_widget]
-            self._main.children[1].children[1].layout.visibility = "visible"
-        elif self._options.value == "Constant":
-            self._main.children[1].children = [self._options, self._constant]
-            self._main.children[1].children[1].layout.visibility = "visible"
-        else:
-            self._main.children[1].children[1].layout.visibility = "hidden"
-
-    def objects_setter(self, _):
-        if self.objects.value is not None:
-            self.options.value = "Model"
-
-    @property
-    def constant(self):
-        return self._constant
-
-    @property
-    def description(self):
-        return self._description
-
-    @property
-    def identifier(self):
-        return self._identifier
-
-    @property
-    def options(self):
-        return self._options
-
-    @property
-    def units(self):
-        return self._units
-
-    @units.setter
-    def units(self, value):
-        self._units = value
-        self._constant.description = value
