@@ -5,11 +5,14 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 from uuid import UUID
 
 from geoh5py.objects.surveys.electromagnetics.magnetotellurics import MTReceivers
 
 from geoapps.inversion import default_ui_json as base_default_ui_json
+from geoapps.inversion.constants import validations as base_validations
 
 ################# defaults ##################
 
@@ -21,6 +24,13 @@ inversion_defaults = {
     "topography_object": None,
     "topography": None,
     "data_object": None,
+    "resolution": None,
+    "z_from_topo": False,
+    "receivers_radar_drape": None,
+    "receivers_offset_x": None,
+    "receivers_offset_y": None,
+    "receivers_offset_z": None,
+    "gps_receivers_offset": None,
     "zxx_real_channel": None,
     "zxx_real_uncertainty": None,
     "zxx_imag_channel": None,
@@ -37,33 +47,16 @@ inversion_defaults = {
     "zyy_real_uncertainty": None,
     "zyy_imag_channel": None,
     "zyy_imag_uncertainty": None,
-    "starting_model_object": None,
-    "starting_model": 1e-3,
+    "mesh": None,
     "background_conductivity": 1e-3,
-    "tile_spatial": 1,
+    "starting_model": 1e-3,
+    "reference_model": 1e-3,
+    "lower_bound": None,
+    "upper_bound": None,
     "output_tile_files": False,
-    "z_from_topo": False,
-    "receivers_radar_drape": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
-    "receivers_offset_z": 0.0,
-    "gps_receivers_offset": None,
     "ignore_values": None,
-    "resolution": None,
     "detrend_order": None,
     "detrend_type": None,
-    "max_chunk_size": 128,
-    "chunk_by_rows": True,
-    "mesh": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "max_distance": 5000.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
     "window_center_x": None,
     "window_center_y": None,
     "window_width": None,
@@ -71,23 +64,13 @@ inversion_defaults = {
     "window_azimuth": None,
     "inversion_style": "voxel",
     "chi_factor": 1.0,
-    "sens_wts_threshold": 60.0,
-    "every_iteration_bool": False,
-    "f_min_change": 1e-4,
-    "minGNiter": 1,
-    "beta_tol": 0.5,
-    "prctile": 95,
-    "coolingRate": 1,
-    "coolEps_q": True,
-    "coolEpsFact": 1.2,
-    "beta_search": False,
-    "starting_chi_factor": None,
-    "max_iterations": 25,
-    "max_line_search_iterations": 20,
-    "max_cg_iterations": 30,
-    "max_global_iterations": 100,
     "initial_beta_ratio": 1e2,
     "initial_beta": None,
+    "coolingRate": 2,
+    "coolingFactor": 2.0,
+    "max_global_iterations": 50,
+    "max_line_search_iterations": 20,
+    "max_cg_iterations": 30,
     "tol_cg": 1e-4,
     "alpha_s": 1.0,
     "alpha_x": 1.0,
@@ -97,16 +80,24 @@ inversion_defaults = {
     "x_norm": 2.0,
     "y_norm": 2.0,
     "z_norm": 2.0,
-    "reference_model_object": None,
-    "reference_model": 1e-3,
     "gradient_type": "total",
-    "lower_bound_object": None,
-    "lower_bound": None,
-    "upper_bound_object": None,
-    "upper_bound": None,
+    "max_irls_iterations": 25,
+    "starting_chi_factor": None,
+    "f_min_change": 1e-4,
+    "beta_tol": 0.5,
+    "prctile": 95,
+    "coolEps_q": True,
+    "coolEpsFact": 1.2,
+    "beta_search": False,
+    "sens_wts_threshold": 60.0,
+    "every_iteration_bool": False,
     "parallelized": True,
     "n_cpu": None,
+    "tile_spatial": 1,
+    "store_sensitivities": "disk",
     "max_ram": None,
+    "max_chunk_size": 128,
+    "chunk_by_rows": True,
     "out_group": "MagnetotelluricsInversion",
     "monitoring_directory": None,
     "workspace_geoh5": None,
@@ -132,6 +123,13 @@ forward_defaults = {
     "topography_object": None,
     "topography": None,
     "data_object": None,
+    "resolution": None,
+    "z_from_topo": False,
+    "receivers_radar_drape": None,
+    "receivers_offset_x": None,
+    "receivers_offset_y": None,
+    "receivers_offset_z": None,
+    "gps_receivers_offset": None,
     "zxx_real_channel_bool": False,
     "zxx_imag_channel_bool": False,
     "zxy_real_channel_bool": False,
@@ -140,30 +138,10 @@ forward_defaults = {
     "zyx_imag_channel_bool": False,
     "zyy_real_channel_bool": False,
     "zyy_imag_channel_bool": False,
-    "starting_model_object": None,
-    "starting_model": 1e-3,
-    "background_conductivity": 1e-3,
-    "tile_spatial": 1,
-    "output_tile_files": False,
-    "z_from_topo": False,
-    "receivers_radar_drape": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
-    "receivers_offset_z": 0.0,
-    "gps_receivers_offset": None,
-    "resolution": None,
-    "max_chunk_size": 128,
-    "chunk_by_rows": True,
     "mesh": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "octree_levels_topo": [16, 8, 4, 2],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "max_distance": 5000.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
+    "background_conductivity": 1e-3,
+    "starting_model": 1e-3,
+    "output_tile_files": False,
     "window_center_x": None,
     "window_center_y": None,
     "window_width": None,
@@ -171,6 +149,9 @@ forward_defaults = {
     "window_azimuth": None,
     "parallelized": True,
     "n_cpu": None,
+    "tile_spatial": 1,
+    "max_chunk_size": 128,
+    "chunk_by_rows": True,
     "out_group": "MagnetotelluricsForward",
     "monitoring_directory": None,
     "workspace_geoh5": None,
@@ -225,7 +206,7 @@ default_ui_json = {
     "zxx_real_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Zxx real",
+        "label": "Zxx real",
         "value": False,
     },
     "zxx_real_channel": {
@@ -234,7 +215,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zxx real channel",
+        "label": "Zxx real",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -246,7 +227,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zxx real uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "zxx_real_channel",
         "dependencyType": "enabled",
@@ -255,7 +236,7 @@ default_ui_json = {
     "zxx_imag_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Zxx imaginary",
+        "label": "Zxx imaginary",
         "value": False,
     },
     "zxx_imag_channel": {
@@ -264,7 +245,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zxx imaginary channel",
+        "label": "Zxx imaginary",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -276,7 +257,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zxx imaginary uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "zxx_imag_channel",
         "dependencyType": "enabled",
@@ -285,7 +266,7 @@ default_ui_json = {
     "zxy_real_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Zxy real",
+        "label": "Zxy real",
         "value": False,
     },
     "zxy_real_channel": {
@@ -294,7 +275,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zxy real channel",
+        "label": "Zxy real",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -306,7 +287,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zxy real uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "zxy_real_channel",
         "dependencyType": "enabled",
@@ -315,7 +296,7 @@ default_ui_json = {
     "zxy_imag_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Zxy imaginary",
+        "label": "Zxy imaginary",
         "value": False,
     },
     "zxy_imag_channel": {
@@ -324,7 +305,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zxy imaginary channel",
+        "label": "Zxy imaginary",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -336,7 +317,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zxy imaginary uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "zxy_imag_channel",
         "dependencyType": "enabled",
@@ -345,7 +326,7 @@ default_ui_json = {
     "zyx_real_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Zyx real",
+        "label": "Zyx real",
         "value": False,
     },
     "zyx_real_channel": {
@@ -354,7 +335,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zyx real channel",
+        "label": "Zyx real",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -366,7 +347,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zyx real uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "zyx_real_channel",
         "dependencyType": "enabled",
@@ -375,7 +356,7 @@ default_ui_json = {
     "zyx_imag_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Zyx imaginary",
+        "label": "Zyx imaginary",
         "value": False,
     },
     "zyx_imag_channel": {
@@ -384,7 +365,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zyx imaginary channel",
+        "label": "Zyx imaginary",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -396,7 +377,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zyx imaginary uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "zyx_imag_channel",
         "dependencyType": "enabled",
@@ -405,7 +386,7 @@ default_ui_json = {
     "zyy_real_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Zyy real",
+        "label": "Zyy real",
         "value": False,
     },
     "zyy_real_channel": {
@@ -414,7 +395,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zyy real channel",
+        "label": "Zyy real",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -426,7 +407,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zyy real uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "zyy_real_channel",
         "dependencyType": "enabled",
@@ -435,7 +416,7 @@ default_ui_json = {
     "zyy_imag_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Zyy imaginary",
+        "label": "Zyy imaginary",
         "value": False,
     },
     "zyy_imag_channel": {
@@ -444,7 +425,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zyy imaginary channel",
+        "label": "Zyy imaginary",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -456,7 +437,7 @@ default_ui_json = {
         "group": "Data",
         "dataGroupType": "Multi-element",
         "main": True,
-        "label": "Zyy imaginary uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "zyy_imag_channel",
         "dependencyType": "enabled",
@@ -465,24 +446,61 @@ default_ui_json = {
     "starting_model": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Starting Model",
+        "group": "Mesh and Models",
         "main": True,
         "isValue": False,
-        "parent": "starting_model_object",
-        "label": "Conductivity (Siemens/m)",
+        "parent": "mesh",
+        "label": "Initial Conductivity (S/m)",
         "property": None,
-        "value": 0.0,
+        "value": 1e-3,
+    },
+    "reference_model": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "main": True,
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Reference Conductivity (S/m)",
+        "property": None,
+        "value": 1e-3,
     },
     "background_conductivity": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Starting Model",
+        "group": "Mesh and Models",
         "main": True,
         "isValue": True,
-        "parent": "starting_model_object",
-        "label": "Background conductivity (Siemens/m)",
+        "parent": "mesh",
+        "label": "Background conductivity (S/m)",
         "property": None,
-        "value": 0.0,
+        "value": 1e-3,
+    },
+    "lower_bound": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Lower bound (S/m)",
+        "property": None,
+        "optional": True,
+        "value": 1e-8,
+        "enabled": False,
+    },
+    "upper_bound": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Upper bound (S/m)",
+        "property": None,
+        "optional": True,
+        "value": 100.0,
+        "enabled": False,
     },
     "out_group": {"label": "Results group name", "value": "MagnetotelluricsInversion"},
 }
@@ -515,33 +533,32 @@ validations = {
     "zyy_imag_channel": {"one_of": "data_channel"},
     "zyy_imag_uncertainty": {"one_of": "uncertainty_channel"},
 }
+validations = dict(base_validations, **validations)
 
 app_initializer = {
     "geoh5": "../../../assets/FlinFlon_natural_sources.geoh5",
     "topography_object": UUID("{cfabb8dd-d1ad-4c4e-a87c-7b3dd224c3f5}"),
-    "data_object": UUID("{c9643051-e623-42d5-b33b-dbc6b2b5b876}"),
-    "zxx_real_channel": UUID("{bd103ff8-5cc1-4acc-a23b-3dd9b875c20f}"),
-    "zxx_real_uncertainty": UUID("{684297aa-f09d-41ab-9c4c-bd9085818e3e}"),
-    "zxx_imag_channel": UUID("{6d463e8b-0e27-4969-bddb-6af21983c469}"),
-    "zxx_imag_uncertainty": UUID("{4f514be9-3420-49ff-8e71-cb44160dfd83}"),
-    "zxy_real_channel": UUID("{89cb9fc3-1220-4649-80a5-fdcaef8e398e}"),
-    "zxy_real_uncertainty": UUID("{79de7a3b-6505-4ee3-aa3f-53966551367c}"),
-    "zxy_imag_channel": UUID("{7fea3bf2-4309-4612-a7f2-6e04ffd4f4ea}"),
-    "zxy_imag_uncertainty": UUID("{fa4a4311-4059-4885-b2f0-a787f9837a47}"),
-    "zyx_real_channel": UUID("{33419705-6555-475c-872c-37fcf69aa351}"),
-    "zyx_real_uncertainty": UUID("{45e0fe91-e18d-42b3-80f6-59235b93ec32}"),
-    "zyx_imag_channel": UUID("{e294ddcd-4a87-4285-8cc6-0a92c0345a4a}"),
-    "zyx_imag_uncertainty": UUID("{07b9bb1f-3b2a-4fba-be09-6f69ddd9b66f}"),
-    "zyy_real_channel": UUID("{fe88daca-060c-416f-8011-7fae78fe7582}"),
-    "zyy_real_uncertainty": UUID("{cbf999bc-f9b9-4c0c-8198-9dac8135e3a3}"),
-    "zyy_imag_channel": UUID("{a8e3f100-43ad-4b37-a56a-34c6cae29f14}"),
-    "zyy_imag_uncertainty": UUID("{52212149-d873-4adc-aae1-08414d7f9a9b}"),
+    "data_object": UUID("{9664afc1-cbda-4955-b936-526ca771f517}"),
+    "zxx_real_channel": UUID("{a73159fc-8c1b-411a-b435-12a5dac4a209}"),
+    "zxx_real_uncertainty": UUID("{e752e8d8-e8e3-4575-b20c-bc2d37cbd269}"),
+    "zxx_imag_channel": UUID("{46271e74-9573-4cd6-8bcb-4c45495fe539}"),
+    "zxx_imag_uncertainty": UUID("{73f77c42-ab78-4972-bb69-b16c990bf7dc}"),
+    "zxy_real_channel": UUID("{40bdf2a1-237f-49e4-baa8-a7c0785f369a}"),
+    "zxy_real_uncertainty": UUID("{8802e943-354f-4ce4-a81f-dde9ef08b8ec}"),
+    "zxy_imag_channel": UUID("{1a135542-b2be-4096-9629-a0bc4357970d}"),
+    "zxy_imag_uncertainty": UUID("{fac85198-cbd2-4510-bce7-12b4b5fcae2f}"),
+    "zyx_real_channel": UUID("{21e6737d-de1a-4af4-9c92-aeeeb6eecf34}"),
+    "zyx_real_uncertainty": UUID("{08141050-365c-40aa-bcfb-54841c9492ce}"),
+    "zyx_imag_channel": UUID("{f1d2750a-99bf-4876-833b-19b9f46124a4}"),
+    "zyx_imag_uncertainty": UUID("{2664535c-295a-4e2a-b403-2a57a821fe08}"),
+    "zyy_real_channel": UUID("{9b7f06e9-5bfb-4a5e-ba90-9cec9990d7d5}"),
+    "zyy_real_uncertainty": UUID("{61d1a3e9-f7ff-4fd8-bc61-2d1b24b9adc6}"),
+    "zyy_imag_channel": UUID("{c9133116-043b-40d9-853d-21f6357f927f}"),
+    "zyy_imag_uncertainty": UUID("{11ebb4f3-eacf-4558-b240-b958526dd273}"),
+    "mesh": UUID("{1200396b-bc4a-4519-85e1-558c2dcac1dd}"),
     "starting_model": 0.0003,
     "reference_model": 0.0003,
     "background_conductivity": 0.0003,
-    "u_cell_size": 100.0,
-    "v_cell_size": 100.0,
-    "w_cell_size": 100.0,
     "resolution": 200.0,
     "window_center_x": None,
     "window_center_y": None,
@@ -559,6 +576,5 @@ app_initializer = {
     "z_norm": 2.0,
     "upper_bound": 100.0,
     "lower_bound": 1e-5,
-    "max_iterations": 15,
-    "out_group": "MTInversion",
+    "max_global_iterations": 50,
 }
