@@ -12,11 +12,11 @@ from uuid import UUID
 
 import numpy as np
 from geoh5py.data import NumericData
-from geoh5py.groups import ContainerGroup
+from geoh5py.groups import SimPEGGroup
 from geoh5py.ui_json import InputFile
 from geoh5py.workspace import Workspace
 
-from geoapps.base.params import BaseParams
+from geoapps.driver_base.params import BaseParams
 
 
 class InversionBaseParams(BaseParams):
@@ -33,12 +33,15 @@ class InversionBaseParams(BaseParams):
     _inversion_type = None
     _ga_group = None
 
-    def __init__(self, input_file=None, forward_only=False, **kwargs):
-        self._forward_only: bool = forward_only
+    def __init__(
+        self, input_file: InputFile | None = None, forward_only: bool = False, **kwargs
+    ):
+        self._forward_only: bool = (
+            forward_only if input_file is None else input_file.data["forward_only"]
+        )
         self._topography_object: UUID = None
         self._topography: UUID | float = None
         self._data_object: UUID = None
-        self._starting_model_object: UUID = None
         self._starting_model: UUID | float = None
         self._tile_spatial = None
         self._z_from_topo: bool = None
@@ -55,15 +58,6 @@ class InversionBaseParams(BaseParams):
         self._chunk_by_rows: bool = None
         self._output_tile_files: bool = None
         self._mesh = None
-        self._u_cell_size: float = None
-        self._v_cell_size: float = None
-        self._w_cell_size: float = None
-        self._octree_levels_topo: list[int] = None
-        self._octree_levels_obs: list[int] = None
-        self._depth_core: float = None
-        self._max_distance: float = None
-        self._horizontal_padding: float = None
-        self._vertical_padding: float = None
         self._window_azimuth: float = None
         self._window_center_x: float = None
         self._window_center_y: float = None
@@ -74,18 +68,18 @@ class InversionBaseParams(BaseParams):
         self._sens_wts_threshold: float = None
         self._every_iteration_bool: bool = None
         self._f_min_change: float = None
-        self._minGNiter: float = None
         self._beta_tol: float = None
         self._prctile: float = None
         self._coolingRate: float = None
+        self._coolingFactor: float = None
         self._coolEps_q: bool = None
         self._coolEpsFact: float = None
         self._beta_search: bool = None
         self._starting_chi_factor: float = None
-        self._max_iterations: int = None
+        self._max_irls_iterations: int = None
+        self._max_global_iterations: int = None
         self._max_line_search_iterations: int = None
         self._max_cg_iterations: int = None
-        self._max_global_iterations: int = None
         self._initial_beta: float = None
         self._initial_beta_ratio: float = None
         self._tol_cg: float = None
@@ -97,19 +91,19 @@ class InversionBaseParams(BaseParams):
         self._x_norm: float = None
         self._y_norm: float = None
         self._z_norm: float = None
-        self._reference_model_object: UUID = None
         self._reference_model = None
         self._gradient_type: str = None
-        self._lower_bound_object: UUID = None
         self._lower_bound = None
-        self._upper_bound_object: UUID = None
         self._upper_bound = None
         self._parallelized: bool = None
         self._n_cpu: int = None
         self._max_ram: float = None
+        self._store_sensitivities: str = None
         self._out_group = None
         self._no_data_value: float = None
         self._distributed_workers = None
+        self._documentation: str = None
+        self._icon: str = None
         self._defaults = (
             self.forward_defaults if self.forward_only else self.inversion_defaults
         )
@@ -119,7 +113,9 @@ class InversionBaseParams(BaseParams):
             ui_json.update(
                 self._forward_ui_json if self.forward_only else self._inversion_ui_json
             )
-            ui_json = {k: ui_json[k] for k in self.defaults}  # Re-order using defaults
+            ui_json = {
+                k: ui_json[k] for k in list(self.defaults)
+            }  # Re-order using defaults
             input_file = InputFile(
                 ui_json=ui_json,
                 data=self.defaults,
@@ -129,6 +125,11 @@ class InversionBaseParams(BaseParams):
 
         super().__init__(input_file=input_file, **kwargs)
 
+        if not self.forward_only:
+            for key in self.__dict__:
+                if "channel_bool" in key and getattr(self, key[:-5], None) is not None:
+                    setattr(self, key, True)
+
     def data_channel(self, component: str):
         """Return uuid of data channel."""
         return getattr(self, "_".join([component, "channel"]), None)
@@ -159,16 +160,12 @@ class InversionBaseParams(BaseParams):
         else:
             return None
 
-    def cell_size(self):
-        """Returns core cell size in all 3 dimensions."""
-        return [self.u_cell_size, self.v_cell_size, self.w_cell_size]
-
     def components(self) -> list[str]:
         """Retrieve component names used to index channel and uncertainty data."""
         comps = []
         channels = [
             k.lstrip("_").split("_channel_bool")[0]
-            for k in self.__dict__.keys()
+            for k in self.__dict__
             if "channel_bool" in k
         ]
 
@@ -199,9 +196,9 @@ class InversionBaseParams(BaseParams):
     def offset(self) -> tuple[list[float], UUID]:
         """Returns offset components as list and drape data."""
         offsets = [
-            self.receivers_offset_x,
-            self.receivers_offset_y,
-            self.receivers_offset_z,
+            0 if self.receivers_offset_x is None else self.receivers_offset_x,
+            0 if self.receivers_offset_y is None else self.receivers_offset_y,
+            0 if self.receivers_offset_z is None else self.receivers_offset_z,
         ]
         is_offset = any([(k != 0) for k in offsets])
         offsets = offsets if is_offset else None
@@ -223,6 +220,11 @@ class InversionBaseParams(BaseParams):
             self.z_norm,
         ]
 
+    @property
+    def directive_list(self):
+        """List of directives"""
+        return self._directive_list
+
     @property
     def forward_defaults(self):
         if getattr(self, "_forward_defaults", None) is None:
@@ -273,14 +275,6 @@ class InversionBaseParams(BaseParams):
     def data_object(self, val):
         self.setter_validator("data_object", val, fun=self._uuid_promoter)
 
-    @property
-    def starting_model_object(self):
-        return self._starting_model_object
-
-    @starting_model_object.setter
-    def starting_model_object(self, val):
-        self.setter_validator("starting_model_object", val, fun=self._uuid_promoter)
-
     @property
     def starting_model(self):
         return self._starting_model
@@ -417,78 +411,6 @@ class InversionBaseParams(BaseParams):
     def mesh(self, val):
         self.setter_validator("mesh", val, fun=self._uuid_promoter)
 
-    @property
-    def u_cell_size(self):
-        return self._u_cell_size
-
-    @u_cell_size.setter
-    def u_cell_size(self, val):
-        self.setter_validator("u_cell_size", val)
-
-    @property
-    def v_cell_size(self):
-        return self._v_cell_size
-
-    @v_cell_size.setter
-    def v_cell_size(self, val):
-        self.setter_validator("v_cell_size", val)
-
-    @property
-    def w_cell_size(self):
-        return self._w_cell_size
-
-    @w_cell_size.setter
-    def w_cell_size(self, val):
-        self.setter_validator("w_cell_size", val)
-
-    @property
-    def octree_levels_topo(self):
-        return self._octree_levels_topo
-
-    @octree_levels_topo.setter
-    def octree_levels_topo(self, val):
-        self.setter_validator("octree_levels_topo", val)
-
-    @property
-    def octree_levels_obs(self):
-        return self._octree_levels_obs
-
-    @octree_levels_obs.setter
-    def octree_levels_obs(self, val):
-        self.setter_validator("octree_levels_obs", val)
-
-    @property
-    def depth_core(self):
-        return self._depth_core
-
-    @depth_core.setter
-    def depth_core(self, val):
-        self.setter_validator("depth_core", val)
-
-    @property
-    def max_distance(self):
-        return self._max_distance
-
-    @max_distance.setter
-    def max_distance(self, val):
-        self.setter_validator("max_distance", val)
-
-    @property
-    def horizontal_padding(self):
-        return self._horizontal_padding
-
-    @horizontal_padding.setter
-    def horizontal_padding(self, val):
-        self.setter_validator("horizontal_padding", val)
-
-    @property
-    def vertical_padding(self):
-        return self._vertical_padding
-
-    @vertical_padding.setter
-    def vertical_padding(self, val):
-        self.setter_validator("vertical_padding", val)
-
     @property
     def window_center_x(self):
         return self._window_center_x
@@ -569,14 +491,6 @@ class InversionBaseParams(BaseParams):
     def f_min_change(self, val):
         self.setter_validator("f_min_change", val)
 
-    @property
-    def minGNiter(self):
-        return self._minGNiter
-
-    @minGNiter.setter
-    def minGNiter(self, val):
-        self.setter_validator("minGNiter", val)
-
     @property
     def beta_tol(self):
         return self._beta_tol
@@ -601,6 +515,14 @@ class InversionBaseParams(BaseParams):
     def coolingRate(self, val):
         self.setter_validator("coolingRate", val)
 
+    @property
+    def coolingFactor(self):
+        return self._coolingFactor
+
+    @coolingFactor.setter
+    def coolingFactor(self, val):
+        self.setter_validator("coolingFactor", val)
+
     @property
     def coolEps_q(self):
         return self._coolEps_q
@@ -634,12 +556,20 @@ class InversionBaseParams(BaseParams):
         self.setter_validator("starting_chi_factor", val)
 
     @property
-    def max_iterations(self):
-        return self._max_iterations
+    def max_irls_iterations(self):
+        return self._max_irls_iterations
 
-    @max_iterations.setter
-    def max_iterations(self, val):
-        self.setter_validator("max_iterations", val)
+    @max_irls_iterations.setter
+    def max_irls_iterations(self, val):
+        self.setter_validator("max_irls_iterations", val)
+
+    @property
+    def max_global_iterations(self):
+        return self._max_global_iterations
+
+    @max_global_iterations.setter
+    def max_global_iterations(self, val):
+        self.setter_validator("max_global_iterations", val)
 
     @property
     def max_line_search_iterations(self):
@@ -657,14 +587,6 @@ class InversionBaseParams(BaseParams):
     def max_cg_iterations(self, val):
         self.setter_validator("max_cg_iterations", val)
 
-    @property
-    def max_global_iterations(self):
-        return self._max_global_iterations
-
-    @max_global_iterations.setter
-    def max_global_iterations(self, val):
-        self.setter_validator("max_global_iterations", val)
-
     @property
     def initial_beta(self):
         return self._initial_beta
@@ -753,14 +675,6 @@ class InversionBaseParams(BaseParams):
     def z_norm(self, val):
         self.setter_validator("z_norm", val)
 
-    @property
-    def reference_model_object(self):
-        return self._reference_model_object
-
-    @reference_model_object.setter
-    def reference_model_object(self, val):
-        self.setter_validator("reference_model_object", val, fun=self._uuid_promoter)
-
     @property
     def reference_model(self):
         return self._reference_model
@@ -777,14 +691,6 @@ class InversionBaseParams(BaseParams):
     def gradient_type(self, val):
         self.setter_validator("gradient_type", val)
 
-    @property
-    def lower_bound_object(self):
-        return self._lower_bound_object
-
-    @lower_bound_object.setter
-    def lower_bound_object(self, val):
-        self.setter_validator("lower_bound_object", val, fun=self._uuid_promoter)
-
     @property
     def lower_bound(self):
         return self._lower_bound
@@ -793,14 +699,6 @@ class InversionBaseParams(BaseParams):
     def lower_bound(self, val):
         self.setter_validator("lower_bound", val, fun=self._uuid_promoter)
 
-    @property
-    def upper_bound_object(self):
-        return self._upper_bound_object
-
-    @upper_bound_object.setter
-    def upper_bound_object(self, val):
-        self.setter_validator("upper_bound_object", val, fun=self._uuid_promoter)
-
     @property
     def upper_bound(self):
         return self._upper_bound
@@ -833,6 +731,14 @@ class InversionBaseParams(BaseParams):
     def max_ram(self, val):
         self.setter_validator("max_ram", val)
 
+    @property
+    def store_sensitivities(self):
+        return self._store_sensitivities
+
+    @store_sensitivities.setter
+    def store_sensitivities(self, val):
+        self.setter_validator("store_sensitivities", val)
+
     @property
     def out_group(self):
         return self._out_group
@@ -846,18 +752,18 @@ class InversionBaseParams(BaseParams):
         self.setter_validator(
             "out_group",
             val,
-            fun=lambda x: x.name if isinstance(val, ContainerGroup) else x,
         )
 
     @property
-    def ga_group(self) -> ContainerGroup | None:
+    def ga_group(self) -> SimPEGGroup | None:
         if (
             getattr(self, "_ga_group", None) is None
             and isinstance(self.geoh5, Workspace)
             and isinstance(self.out_group, str)
         ):
-            self._ga_group = ContainerGroup.create(self.geoh5, name=self.out_group)
-
+            self._ga_group = SimPEGGroup.create(self.geoh5, name=self.out_group)
+        elif isinstance(self.out_group, SimPEGGroup):
+            self._ga_group = self.out_group
         return self._ga_group
 
     @property
