@@ -5,24 +5,40 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 from uuid import UUID
 
 from geoh5py.objects import Grid2D, Points, Surface
 
 from geoapps.inversion import default_ui_json as base_default_ui_json
+from geoapps.inversion.constants import validations as base_validations
 
 ################# defaults ##################
 
 inversion_defaults = {
-    "title": "SimPEG gravity inversion",
+    "title": "Gravity inversion",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "gravity",
     "geoh5": None,  # Must remain at top of list for notebook app initialization
     "forward_only": False,
     "topography_object": None,
     "topography": None,
     "data_object": None,
+    "resolution": None,
+    "z_from_topo": False,
+    "receivers_offset_x": None,
+    "receivers_offset_y": None,
+    "receivers_offset_z": None,
+    "receivers_radar_drape": None,
+    "gps_receivers_offset": None,
     "gz_channel": None,
     "gz_uncertainty": 1.0,
+    "gx_channel": None,
+    "gx_uncertainty": 1.0,
+    "gy_channel": None,
+    "gy_uncertainty": 1.0,
     "guv_channel": None,
     "guv_uncertainty": 1.0,
     "gxy_channel": None,
@@ -37,36 +53,15 @@ inversion_defaults = {
     "gxz_uncertainty": 1.0,
     "gyz_channel": None,
     "gyz_uncertainty": 1.0,
-    "gx_channel": None,
-    "gx_uncertainty": 1.0,
-    "gy_channel": None,
-    "gy_uncertainty": 1.0,
-    "starting_model_object": None,
-    "starting_model": 1e-4,
-    "tile_spatial": 1,
+    "mesh": None,
+    "starting_model": 1e-3,
+    "reference_model": 0.0,
+    "lower_bound": None,
+    "upper_bound": None,
     "output_tile_files": False,
-    "z_from_topo": False,
-    "receivers_radar_drape": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
-    "receivers_offset_z": 0.0,
-    "gps_receivers_offset": None,
     "ignore_values": None,
-    "resolution": None,
     "detrend_order": None,
     "detrend_type": None,
-    "max_chunk_size": 128,
-    "chunk_by_rows": True,
-    "mesh": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "max_distance": 5000.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
     "window_center_x": None,
     "window_center_y": None,
     "window_width": None,
@@ -74,23 +69,13 @@ inversion_defaults = {
     "window_azimuth": None,
     "inversion_style": "voxel",
     "chi_factor": 1.0,
-    "sens_wts_threshold": 0.0,
-    "every_iteration_bool": False,
-    "f_min_change": 1e-4,
-    "minGNiter": 1,
-    "beta_tol": 0.5,
-    "prctile": 95,
-    "coolingRate": 1,
-    "coolEps_q": True,
-    "coolEpsFact": 1.2,
-    "beta_search": False,
-    "starting_chi_factor": None,
-    "max_iterations": 25,
-    "max_line_search_iterations": 20,
-    "max_cg_iterations": 30,
-    "max_global_iterations": 100,
     "initial_beta_ratio": 10.0,
     "initial_beta": None,
+    "coolingRate": 1,
+    "coolingFactor": 2.0,
+    "max_global_iterations": 100,
+    "max_line_search_iterations": 20,
+    "max_cg_iterations": 30,
     "tol_cg": 1e-4,
     "alpha_s": 1.0,
     "alpha_x": 1.0,
@@ -100,16 +85,24 @@ inversion_defaults = {
     "x_norm": 2.0,
     "y_norm": 2.0,
     "z_norm": 2.0,
-    "reference_model_object": None,
-    "reference_model": 0.0,
     "gradient_type": "total",
-    "lower_bound_object": None,
-    "lower_bound": None,
-    "upper_bound_object": None,
-    "upper_bound": None,
+    "max_irls_iterations": 25,
+    "starting_chi_factor": None,
+    "f_min_change": 1e-4,
+    "beta_tol": 0.5,
+    "prctile": 95,
+    "coolEps_q": True,
+    "coolEpsFact": 1.2,
+    "beta_search": False,
+    "sens_wts_threshold": 1.0,
+    "every_iteration_bool": False,
     "parallelized": True,
     "n_cpu": None,
+    "tile_spatial": 1,
+    "store_sensitivities": "disk",
     "max_ram": None,
+    "max_chunk_size": 128,
+    "chunk_by_rows": True,
     "out_group": "GravityInversion",
     "monitoring_directory": None,
     "workspace_geoh5": None,
@@ -129,14 +122,25 @@ inversion_defaults = {
     "gy_channel_bool": False,
 }
 forward_defaults = {
-    "title": "SimPEG gravity Forward",
+    "title": "Gravity Forward",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "gravity",
     "geoh5": None,  # Must remain at top of list for notebook app initialization
     "forward_only": True,
     "topography_object": None,
     "topography": None,
     "data_object": None,
+    "resolution": None,
+    "z_from_topo": False,
+    "receivers_radar_drape": None,
+    "receivers_offset_x": None,
+    "receivers_offset_y": None,
+    "receivers_offset_z": None,
+    "gps_receivers_offset": None,
     "gz_channel_bool": False,
+    "gx_channel_bool": False,
+    "gy_channel_bool": False,
     "guv_channel_bool": False,
     "gxy_channel_bool": False,
     "gxx_channel_bool": False,
@@ -144,31 +148,9 @@ forward_defaults = {
     "gzz_channel_bool": False,
     "gxz_channel_bool": False,
     "gyz_channel_bool": False,
-    "gx_channel_bool": False,
-    "gy_channel_bool": False,
-    "starting_model_object": None,
-    "starting_model": None,
-    "tile_spatial": 1,
-    "output_tile_files": False,
-    "z_from_topo": False,
-    "receivers_radar_drape": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
-    "receivers_offset_z": 0.0,
-    "gps_receivers_offset": None,
-    "resolution": None,
-    "max_chunk_size": 128,
-    "chunk_by_rows": True,
     "mesh": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "max_distance": 5000.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
+    "starting_model": None,
+    "output_tile_files": False,
     "window_center_x": None,
     "window_center_y": None,
     "window_width": None,
@@ -176,6 +158,9 @@ forward_defaults = {
     "window_azimuth": None,
     "parallelized": True,
     "n_cpu": None,
+    "tile_spatial": 1,
+    "max_chunk_size": 128,
+    "chunk_by_rows": True,
     "out_group": "GravityForward",
     "monitoring_directory": None,
     "workspace_geoh5": None,
@@ -196,6 +181,8 @@ forward_defaults = {
 
 inversion_ui_json = {
     "gz_channel_bool": False,
+    "gx_channel_bool": False,
+    "gy_channel_bool": False,
     "guv_channel_bool": False,
     "gxy_channel_bool": False,
     "gxx_channel_bool": False,
@@ -203,8 +190,6 @@ inversion_ui_json = {
     "gzz_channel_bool": False,
     "gxz_channel_bool": False,
     "gyz_channel_bool": False,
-    "gx_channel_bool": False,
-    "gy_channel_bool": False,
 }
 
 forward_ui_json = {
@@ -220,12 +205,28 @@ forward_ui_json = {
 }
 
 default_ui_json = {
-    "title": "SimPEG gravity inversion",
+    "title": "Gravity inversion",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "gravity",
+    "data_object": {
+        "main": True,
+        "group": "Data",
+        "label": "Object",
+        "meshType": [
+            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
+            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
+            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
+            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
+            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
+            "{4ea87376-3ece-438b-bf12-3479733ded46}",
+        ],
+        "value": None,
+    },
     "gz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Gz",
+        "label": "Gz (mGal)",
         "value": False,
     },
     "gz_channel": {
@@ -233,10 +234,10 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Gz channel",
+        "label": "Gz (mGal)",
         "parent": "data_object",
         "optional": True,
-        "enabled": False,
+        "enabled": True,
         "value": None,
     },
     "gz_uncertainty": {
@@ -245,227 +246,17 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Gz uncertainty (mGal)",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "gz_channel",
         "dependencyType": "enabled",
         "property": None,
         "value": 1.0,
     },
-    "guv_channel_bool": {
-        "group": "Data",
-        "main": True,
-        "label": "Use Guv",
-        "value": False,
-    },
-    "guv_channel": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "label": "Guv channel",
-        "parent": "data_object",
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
-    "guv_uncertainty": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "isValue": True,
-        "label": "Guv uncertainty (mGal)",
-        "parent": "data_object",
-        "dependency": "guv_channel",
-        "dependencyType": "enabled",
-        "property": None,
-        "value": 1.0,
-    },
-    "gxy_channel_bool": {
-        "group": "Data",
-        "main": True,
-        "label": "Use Gxy (Gne)",
-        "value": False,
-    },
-    "gxy_channel": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "label": "Gxy (Gne) channel",
-        "parent": "data_object",
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
-    "gxy_uncertainty": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "isValue": True,
-        "label": "Gxy (Gne) uncertainty (mGal)",
-        "parent": "data_object",
-        "dependency": "gxy_channel",
-        "dependencyType": "enabled",
-        "property": None,
-        "value": 1.0,
-    },
-    "gxx_channel_bool": {
-        "group": "Data",
-        "main": True,
-        "label": "Use Gxx",
-        "value": False,
-    },
-    "gxx_channel": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "label": "Gxx channel",
-        "parent": "data_object",
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
-    "gxx_uncertainty": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "isValue": True,
-        "label": "Gxx uncertainty (mGal)",
-        "parent": "data_object",
-        "dependency": "gxx_channel",
-        "dependencyType": "enabled",
-        "property": None,
-        "value": 1.0,
-    },
-    "gyy_channel_bool": {
-        "group": "Data",
-        "main": True,
-        "label": "Use Gyy",
-        "value": False,
-    },
-    "gyy_channel": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "label": "Gyy channel",
-        "parent": "data_object",
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
-    "gyy_uncertainty": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "isValue": True,
-        "label": "Gyy uncertainty (mGal)",
-        "parent": "data_object",
-        "dependency": "gyy_channel",
-        "dependencyType": "enabled",
-        "property": None,
-        "value": 1.0,
-    },
-    "gzz_channel_bool": {
-        "group": "Data",
-        "main": True,
-        "label": "Use Gzz",
-        "value": False,
-    },
-    "gzz_channel": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "label": "Gzz channel",
-        "parent": "data_object",
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
-    "gzz_uncertainty": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "isValue": True,
-        "label": "Gzz uncertainty (mGal)",
-        "parent": "data_object",
-        "dependency": "gzz_channel",
-        "dependencyType": "enabled",
-        "property": None,
-        "value": 1.0,
-    },
-    "gxz_channel_bool": {
-        "group": "Data",
-        "main": True,
-        "label": "Use Gxz",
-        "value": False,
-    },
-    "gxz_channel": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "label": "Gxz channel",
-        "parent": "data_object",
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
-    "gxz_uncertainty": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "isValue": True,
-        "label": "Gxz uncertainty (mGal)",
-        "parent": "data_object",
-        "dependency": "gxz_channel",
-        "dependencyType": "enabled",
-        "property": None,
-        "value": 1.0,
-    },
-    "gyz_channel_bool": {
-        "group": "Data",
-        "main": True,
-        "label": "Use Gyz",
-        "value": False,
-    },
-    "gyz_channel": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "label": "Gyz channel",
-        "parent": "data_object",
-        "optional": True,
-        "enabled": False,
-        "value": None,
-    },
-    "gyz_uncertainty": {
-        "association": ["Cell", "Vertex"],
-        "dataType": "Float",
-        "group": "Data",
-        "main": True,
-        "isValue": True,
-        "label": "Gyz uncertainty (mGal)",
-        "parent": "data_object",
-        "dependency": "gyz_channel",
-        "dependencyType": "enabled",
-        "property": None,
-        "value": 1.0,
-    },
     "gx_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Gx",
+        "label": "Gx (mGal)",
         "value": False,
     },
     "gx_channel": {
@@ -473,7 +264,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Gx channel",
+        "label": "Gx (mGal)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -485,7 +276,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Gx uncertainty (mGal)",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "gx_channel",
         "dependencyType": "enabled",
@@ -495,7 +286,7 @@ default_ui_json = {
     "gy_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Gy",
+        "label": "Gy (mGal)",
         "value": False,
     },
     "gy_channel": {
@@ -503,7 +294,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Gy channel",
+        "label": "Gy (mGal)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -515,25 +306,272 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Gy uncertainty (mGal)",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "gy_channel",
         "dependencyType": "enabled",
         "property": None,
         "value": 1.0,
     },
+    "guv_channel_bool": {
+        "group": "Data",
+        "main": True,
+        "label": "Guv (Eo)",
+        "value": False,
+    },
+    "guv_channel": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "label": "Guv (Eo)",
+        "parent": "data_object",
+        "optional": True,
+        "enabled": False,
+        "value": None,
+    },
+    "guv_uncertainty": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "isValue": True,
+        "label": "Uncertainty",
+        "parent": "data_object",
+        "dependency": "guv_channel",
+        "dependencyType": "enabled",
+        "property": None,
+        "value": 1.0,
+    },
+    "gxy_channel_bool": {
+        "group": "Data",
+        "main": True,
+        "label": "Gxy/Gne (Eo)",
+        "value": False,
+    },
+    "gxy_channel": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "label": "Gxy/Gne (Eo)",
+        "parent": "data_object",
+        "optional": True,
+        "enabled": False,
+        "value": None,
+    },
+    "gxy_uncertainty": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "isValue": True,
+        "label": "Uncertainty",
+        "parent": "data_object",
+        "dependency": "gxy_channel",
+        "dependencyType": "enabled",
+        "property": None,
+        "value": 1.0,
+    },
+    "gxx_channel_bool": {
+        "group": "Data",
+        "main": True,
+        "label": "Gxx (Eo)",
+        "value": False,
+    },
+    "gxx_channel": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "label": "Gxx (Eo)",
+        "parent": "data_object",
+        "optional": True,
+        "enabled": False,
+        "value": None,
+    },
+    "gxx_uncertainty": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "isValue": True,
+        "label": "Uncertainty",
+        "parent": "data_object",
+        "dependency": "gxx_channel",
+        "dependencyType": "enabled",
+        "property": None,
+        "value": 1.0,
+    },
+    "gyy_channel_bool": {
+        "group": "Data",
+        "main": True,
+        "label": "Gyy (Eo)",
+        "value": False,
+    },
+    "gyy_channel": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "label": "Gyy (Eo)",
+        "parent": "data_object",
+        "optional": True,
+        "enabled": False,
+        "value": None,
+    },
+    "gyy_uncertainty": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "isValue": True,
+        "label": "Uncertainty",
+        "parent": "data_object",
+        "dependency": "gyy_channel",
+        "dependencyType": "enabled",
+        "property": None,
+        "value": 1.0,
+    },
+    "gzz_channel_bool": {
+        "group": "Data",
+        "main": True,
+        "label": "Gzz (Eo)",
+        "value": False,
+    },
+    "gzz_channel": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "label": "Gzz (Eo)",
+        "parent": "data_object",
+        "optional": True,
+        "enabled": False,
+        "value": None,
+    },
+    "gzz_uncertainty": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "isValue": True,
+        "label": "Uncertainty",
+        "parent": "data_object",
+        "dependency": "gzz_channel",
+        "dependencyType": "enabled",
+        "property": None,
+        "value": 1.0,
+    },
+    "gxz_channel_bool": {
+        "group": "Data",
+        "main": True,
+        "label": "Gxz (Eo)",
+        "value": False,
+    },
+    "gxz_channel": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "label": "Gxz (Eo)",
+        "parent": "data_object",
+        "optional": True,
+        "enabled": False,
+        "value": None,
+    },
+    "gxz_uncertainty": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "isValue": True,
+        "label": "Uncertainty",
+        "parent": "data_object",
+        "dependency": "gxz_channel",
+        "dependencyType": "enabled",
+        "property": None,
+        "value": 1.0,
+    },
+    "gyz_channel_bool": {
+        "group": "Data",
+        "main": True,
+        "label": "Gyz (Eo)",
+        "value": False,
+    },
+    "gyz_channel": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "label": "Gyz (Eo)",
+        "parent": "data_object",
+        "optional": True,
+        "enabled": False,
+        "value": None,
+    },
+    "gyz_uncertainty": {
+        "association": ["Cell", "Vertex"],
+        "dataType": "Float",
+        "group": "Data",
+        "main": True,
+        "isValue": True,
+        "label": "Uncertainty",
+        "parent": "data_object",
+        "dependency": "gyz_channel",
+        "dependencyType": "enabled",
+        "property": None,
+        "value": 1.0,
+    },
     "starting_model": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Starting Model",
+        "group": "Mesh and Models",
         "main": True,
         "isValue": True,
-        "parent": "starting_model_object",
-        "label": "Density (g/cc)",
+        "parent": "mesh",
+        "label": "Initial Density (g/cc)",
+        "property": None,
+        "value": 1e-3,
+    },
+    "reference_model": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Reference Density (g/cc)",
         "property": None,
         "value": 0.0,
     },
-    "out_group": {"label": "Results group name", "value": "gravity"},
+    "lower_bound": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Lower bound (g/cc)",
+        "property": None,
+        "optional": True,
+        "value": -10.0,
+        "enabled": False,
+    },
+    "upper_bound": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Upper bound (g/cc)",
+        "property": None,
+        "optional": True,
+        "value": 10.0,
+        "enabled": False,
+    },
+    "out_group": {"label": "Results group name", "value": "GravityInversion"},
 }
 
 default_ui_json = dict(base_default_ui_json, **default_ui_json)
@@ -569,34 +607,38 @@ validations = {
     "gy_uncertainty": {"one_of": "uncertainty channel"},
 }
 
+validations = dict(base_validations, **validations)
+
 app_initializer = {
     "geoh5": "../../../assets/FlinFlon.geoh5",
     "forward_only": False,
     "data_object": UUID("{538a7eb1-2218-4bec-98cc-0a759aa0ef4f}"),
-    "gz_channel": UUID("{6de9177a-8277-4e17-b76c-2b8b05dcf23c}"),
-    "gz_uncertainty": 0.05,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
+    "gxx_channel": UUID("{53e59b2b-c2ae-4b77-923b-23e06d874e62}"),
+    "gxx_uncertainty": 1.0,
+    "gyy_channel": UUID("{51c0acd7-84b8-421f-a66b-fdc15c826a47}"),
+    "gyy_uncertainty": 1.0,
+    "gzz_channel": UUID("{f450906d-80e2-4c50-ab27-6da5cf0906af}"),
+    "gzz_uncertainty": 1.0,
+    "gxy_channel": UUID("{9c2afb52-d7b6-4a21-88e9-23bfe9459529}"),
+    "gxy_uncertainty": 1.0,
+    "gxz_channel": UUID("{55a38ea9-ab20-4944-9fe0-3f77b1f4dcc2}"),
+    "gxz_uncertainty": 1.0,
+    "gyz_channel": UUID("{3d19bd53-8bb8-4634-aeae-4e3a90e9d19e}"),
+    "gyz_uncertainty": 1.0,
+    "mesh": UUID("{385f341f-1027-4b8e-9a86-93be239aa3fb}"),
     "resolution": 50.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 1200.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
-    "window_center_x": 314155.0,
-    "window_center_y": 6071000.0,
+    "window_center_x": 314565.0,
+    "window_center_y": 6072334.0,
     "window_width": 1000.0,
-    "window_height": 1000.0,
+    "window_height": 1500.0,
     "window_azimuth": 0.0,
     "s_norm": 0.0,
     "x_norm": 2.0,
     "y_norm": 2.0,
     "z_norm": 2.0,
-    "starting_model": 1e-4,
-    "max_iterations": 25,
+    "starting_model": 1e-3,
+    "max_global_iterations": 25,
     "topography_object": UUID("{ab3c2083-6ea8-4d31-9230-7aad3ec09525}"),
     "topography": UUID("{a603a762-f6cb-4b21-afda-3160e725bf7d}"),
     "z_from_topo": True,
-    "out_group": "GravityInversion",
 }
