@@ -5,16 +5,21 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 from uuid import UUID
 
 from geoh5py.objects import Grid2D, Points, Surface
 
 from geoapps.inversion import default_ui_json as base_default_ui_json
+from geoapps.inversion.constants import validations as base_validations
 
 ################# defaults ##################
 
 inversion_defaults = {
-    "title": "SimPEG Magnetic Susceptibility inversion",
+    "title": "Magnetic Susceptibility inversion",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "magnetic scalar",
     "geoh5": None,  # Must remain at top of list for notebook app initialization
     "forward_only": False,
@@ -24,8 +29,21 @@ inversion_defaults = {
     "topography_object": None,
     "topography": None,
     "data_object": None,
+    "resolution": None,
+    "z_from_topo": False,
+    "receivers_offset_x": None,
+    "receivers_offset_y": None,
+    "receivers_offset_z": None,
+    "receivers_radar_drape": None,
+    "gps_receivers_offset": None,
     "tmi_channel": None,
     "tmi_uncertainty": 1.0,
+    "bx_channel": None,
+    "bx_uncertainty": 1.0,
+    "by_channel": None,
+    "by_uncertainty": 1.0,
+    "bz_channel": None,
+    "bz_uncertainty": 1.0,
     "bxx_channel": None,
     "bxx_uncertainty": 1.0,
     "bxy_channel": None,
@@ -38,38 +56,15 @@ inversion_defaults = {
     "byz_uncertainty": 1.0,
     "bzz_channel": None,
     "bzz_uncertainty": 1.0,
-    "bx_channel": None,
-    "bx_uncertainty": 1.0,
-    "by_channel": None,
-    "by_uncertainty": 1.0,
-    "bz_channel": None,
-    "bz_uncertainty": 1.0,
-    "starting_model_object": None,
+    "mesh": None,
     "starting_model": 1e-4,
-    "tile_spatial": 1,
+    "reference_model": 0.0,
+    "lower_bound": None,
+    "upper_bound": None,
     "output_tile_files": False,
-    "z_from_topo": False,
-    "receivers_radar_drape": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
-    "receivers_offset_z": 0.0,
-    "gps_receivers_offset": None,
     "ignore_values": None,
-    "resolution": None,
     "detrend_order": None,
     "detrend_type": None,
-    "max_chunk_size": 128,
-    "chunk_by_rows": True,
-    "mesh": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "max_distance": 5000.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
     "window_center_x": None,
     "window_center_y": None,
     "window_width": None,
@@ -77,23 +72,13 @@ inversion_defaults = {
     "window_azimuth": None,
     "inversion_style": "voxel",
     "chi_factor": 1.0,
-    "sens_wts_threshold": 0.0,
-    "every_iteration_bool": False,
-    "f_min_change": 1e-4,
-    "minGNiter": 1,
-    "beta_tol": 0.5,
-    "prctile": 95,
-    "coolingRate": 1,
-    "coolEps_q": True,
-    "coolEpsFact": 1.2,
-    "beta_search": False,
-    "starting_chi_factor": None,
-    "max_iterations": 25,
-    "max_line_search_iterations": 20,
-    "max_cg_iterations": 30,
-    "max_global_iterations": 100,
     "initial_beta_ratio": 10.0,
     "initial_beta": None,
+    "coolingRate": 1,
+    "coolingFactor": 2.0,
+    "max_global_iterations": 100,
+    "max_line_search_iterations": 20,
+    "max_cg_iterations": 30,
     "tol_cg": 1e-4,
     "alpha_s": 1.0,
     "alpha_x": 1.0,
@@ -103,17 +88,25 @@ inversion_defaults = {
     "x_norm": 2.0,
     "y_norm": 2.0,
     "z_norm": 2.0,
-    "reference_model_object": None,
-    "reference_model": 0.0,
     "gradient_type": "total",
-    "lower_bound_object": None,
-    "lower_bound": None,
-    "upper_bound_object": None,
-    "upper_bound": None,
+    "max_irls_iterations": 25,
+    "starting_chi_factor": None,
+    "f_min_change": 1e-4,
+    "beta_tol": 0.5,
+    "prctile": 95,
+    "coolEps_q": True,
+    "coolEpsFact": 1.2,
+    "beta_search": False,
+    "sens_wts_threshold": 1.0,
+    "every_iteration_bool": False,
     "parallelized": True,
     "n_cpu": None,
+    "tile_spatial": 1,
+    "store_sensitivities": "disk",
     "max_ram": None,
-    "out_group": "SusceptibilityInversion",
+    "max_chunk_size": 128,
+    "chunk_by_rows": True,
+    "out_group": "MagneticSusceptibilityInversion",
     "monitoring_directory": None,
     "workspace_geoh5": None,
     "run_command": "geoapps.inversion.driver",
@@ -132,7 +125,9 @@ inversion_defaults = {
     "bz_channel_bool": False,
 }
 forward_defaults = {
-    "title": "SimPEG Magnetic Susceptibility Forward",
+    "title": "Magnetic Susceptibility Forward",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "magnetic scalar",
     "geoh5": None,  # Must remain at top of list for notebook app initialization
     "forward_only": True,
@@ -142,39 +137,26 @@ forward_defaults = {
     "topography_object": None,
     "topography": None,
     "data_object": None,
+    "resolution": 50.0,
+    "z_from_topo": False,
+    "receivers_offset_x": None,
+    "receivers_offset_y": None,
+    "receivers_offset_z": None,
+    "receivers_radar_drape": None,
+    "gps_receivers_offset": None,
     "tmi_channel_bool": False,
+    "bx_channel_bool": False,
+    "by_channel_bool": False,
+    "bz_channel_bool": False,
     "bxx_channel_bool": False,
     "bxy_channel_bool": False,
     "bxz_channel_bool": False,
     "byy_channel_bool": False,
     "byz_channel_bool": False,
     "bzz_channel_bool": False,
-    "bx_channel_bool": False,
-    "by_channel_bool": False,
-    "bz_channel_bool": False,
-    "starting_model_object": None,
-    "starting_model": None,
-    "tile_spatial": 1,
-    "output_tile_files": False,
-    "z_from_topo": False,
-    "receivers_radar_drape": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
-    "receivers_offset_z": 0.0,
-    "gps_receivers_offset": None,
-    "resolution": None,
-    "max_chunk_size": 128,
-    "chunk_by_rows": True,
     "mesh": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "max_distance": 5000.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
+    "starting_model": 1e-4,
+    "output_tile_files": False,
     "window_center_x": None,
     "window_center_y": None,
     "window_width": None,
@@ -182,6 +164,9 @@ forward_defaults = {
     "window_azimuth": None,
     "parallelized": True,
     "n_cpu": None,
+    "tile_spatial": 1,
+    "max_chunk_size": 128,
+    "chunk_by_rows": True,
     "out_group": "MagneticScalarForward",
     "monitoring_directory": None,
     "workspace_geoh5": None,
@@ -202,15 +187,15 @@ forward_defaults = {
 
 inversion_ui_json = {
     "tmi_channel_bool": False,
+    "bx_channel_bool": False,
+    "by_channel_bool": False,
+    "bz_channel_bool": False,
     "bxx_channel_bool": False,
     "bxy_channel_bool": False,
     "bxz_channel_bool": False,
     "byy_channel_bool": False,
     "byz_channel_bool": False,
     "bzz_channel_bool": False,
-    "bx_channel_bool": False,
-    "by_channel_bool": False,
-    "bz_channel_bool": False,
 }
 
 forward_ui_json = {
@@ -226,33 +211,53 @@ forward_ui_json = {
 }
 
 default_ui_json = {
-    "title": "SimPEG Magnetic Susceptibility inversion",
+    "title": "Magnetic Susceptibility inversion",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "magnetic scalar",
     "inducing_field_strength": {
-        "min": 0.0,
+        "min": 1.0,
         "main": True,
         "group": "Inducing Field",
-        "label": "Strength",
+        "label": "Strength (nT)",
         "value": 50000.0,
     },
     "inducing_field_inclination": {
         "min": -90.0,
         "max": 90.0,
+        "precision": 2,
         "main": True,
         "group": "Inducing Field",
-        "label": "Inclination",
+        "label": "Inclination (degrees from North)",
         "value": 90.0,
     },
     "inducing_field_declination": {
+        "min": -180.0,
+        "max": 180.0,
+        "precision": 2,
         "main": True,
         "group": "Inducing Field",
-        "label": "Declination",
+        "label": "Declination (degrees from horizontal)",
         "value": 0.0,
     },
+    "data_object": {
+        "main": True,
+        "group": "Data",
+        "label": "Object",
+        "meshType": [
+            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
+            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
+            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
+            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
+            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
+            "{4ea87376-3ece-438b-bf12-3479733ded46}",
+        ],
+        "value": None,
+    },
     "tmi_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use TMI",
+        "label": "TMI (nT)",
         "value": False,
     },
     "tmi_channel": {
@@ -260,10 +265,10 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "TMI channel",
+        "label": "TMI (nT)",
         "parent": "data_object",
         "optional": True,
-        "enabled": False,
+        "enabled": True,
         "value": None,
     },
     "tmi_uncertainty": {
@@ -272,7 +277,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "TMI uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "tmi_channel",
         "dependencyType": "enabled",
@@ -282,7 +287,7 @@ default_ui_json = {
     "bxx_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bxx",
+        "label": "Bxx (nT/m)",
         "value": False,
     },
     "bxx_channel": {
@@ -290,7 +295,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bxx channel",
+        "label": "Bxx (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -302,7 +307,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bxx uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bxx_channel",
         "dependencyType": "enabled",
@@ -312,7 +317,7 @@ default_ui_json = {
     "bxy_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bxy",
+        "label": "Bxy (nT/m)",
         "value": False,
     },
     "bxy_channel": {
@@ -320,7 +325,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bxy channel",
+        "label": "Bxy (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -332,7 +337,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bxy uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bxy_channel",
         "dependencyType": "enabled",
@@ -342,7 +347,7 @@ default_ui_json = {
     "bxz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bxz",
+        "label": "Bxz (nT/m)",
         "value": False,
     },
     "bxz_channel": {
@@ -350,7 +355,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bxz channel",
+        "label": "Bxz (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -362,7 +367,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bxz uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bxz_channel",
         "dependencyType": "enabled",
@@ -372,7 +377,7 @@ default_ui_json = {
     "byy_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Byy",
+        "label": "Byy (nT/m)",
         "value": False,
     },
     "byy_channel": {
@@ -380,7 +385,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Byy channel",
+        "label": "Byy (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -392,7 +397,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Byy uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "byy_channel",
         "dependencyType": "enabled",
@@ -402,7 +407,7 @@ default_ui_json = {
     "byz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Byz",
+        "label": "Byz (nT/m)",
         "value": False,
     },
     "byz_channel": {
@@ -410,7 +415,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Byz channel",
+        "label": "Byz (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -422,7 +427,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Byz uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "byz_channel",
         "dependencyType": "enabled",
@@ -432,7 +437,7 @@ default_ui_json = {
     "bzz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bzz",
+        "label": "Bzz (nT/m)",
         "value": False,
     },
     "bzz_channel": {
@@ -440,7 +445,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bzz channel",
+        "label": "Bzz (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -452,7 +457,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bzz uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bzz_channel",
         "dependencyType": "enabled",
@@ -462,7 +467,7 @@ default_ui_json = {
     "bx_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bx",
+        "label": "Bx (nT)",
         "value": False,
     },
     "bx_channel": {
@@ -470,7 +475,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bx channel",
+        "label": "Bx (nT)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -482,7 +487,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bx uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bx_channel",
         "dependencyType": "enabled",
@@ -492,7 +497,7 @@ default_ui_json = {
     "by_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use By",
+        "label": "By (nT)",
         "value": False,
     },
     "by_channel": {
@@ -500,7 +505,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "By channel",
+        "label": "By (nT)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -512,7 +517,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "By uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "by_channel",
         "dependencyType": "enabled",
@@ -522,7 +527,7 @@ default_ui_json = {
     "bz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bz",
+        "label": "Bz (nT)",
         "value": False,
     },
     "bz_channel": {
@@ -530,7 +535,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bz channel",
+        "label": "Bz (nT)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -542,7 +547,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bz uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bz_channel",
         "dependencyType": "enabled",
@@ -552,15 +557,55 @@ default_ui_json = {
     "starting_model": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Starting Model",
+        "group": "Mesh and Models",
         "main": True,
         "isValue": True,
-        "parent": "starting_model_object",
-        "label": "Susceptibility (SI)",
+        "parent": "mesh",
+        "label": "Initial Susceptibility (SI)",
         "property": None,
         "value": 1e-4,
     },
-    "out_group": {"label": "Results group name", "value": "SusceptibilityInversion"},
+    "reference_model": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Reference Susceptibility (SI)",
+        "property": None,
+        "value": 0.0,
+    },
+    "lower_bound": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Lower bound (SI)",
+        "property": None,
+        "optional": True,
+        "value": 0.0,
+        "enabled": False,
+    },
+    "upper_bound": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Upper bound (SI)",
+        "property": None,
+        "optional": True,
+        "value": 1.0,
+        "enabled": False,
+    },
+    "out_group": {
+        "label": "Results group name",
+        "value": "MagneticSusceptibilityInversion",
+    },
 }
 
 default_ui_json = dict(base_default_ui_json, **default_ui_json)
@@ -596,24 +641,19 @@ validations = {
     "bz_uncertainty": {"one_of": "uncertainty channel"},
 }
 
+validations = dict(base_validations, **validations)
+
 app_initializer = {
     "geoh5": "../../assets/FlinFlon.geoh5",
     "forward_only": False,
     "data_object": UUID("{538a7eb1-2218-4bec-98cc-0a759aa0ef4f}"),
+    "resolution": 50.0,
     "tmi_channel": UUID("{44822654-b6ae-45b0-8886-2d845f80f422}"),
     "tmi_uncertainty": 10.0,
+    "mesh": UUID("{385f341f-1027-4b8e-9a86-93be239aa3fb}"),
     "inducing_field_strength": 60000.0,
     "inducing_field_inclination": 79.0,
     "inducing_field_declination": 11.0,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "resolution": 50.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
     "window_center_x": 314600.0,
     "window_center_y": 6072300.0,
     "window_width": 1000.0,
@@ -624,12 +664,11 @@ app_initializer = {
     "y_norm": 2.0,
     "z_norm": 2.0,
     "starting_model": 1e-4,
-    "max_iterations": 25,
+    "max_global_iterations": 25,
     "topography_object": UUID("{ab3c2083-6ea8-4d31-9230-7aad3ec09525}"),
     "topography": UUID("{a603a762-f6cb-4b21-afda-3160e725bf7d}"),
     "z_from_topo": True,
     "receivers_offset_x": 0.0,
     "receivers_offset_y": 0.0,
     "receivers_offset_z": 60.0,
-    "out_group": "ScalarInversion",
 }
