@@ -5,16 +5,21 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 from uuid import UUID
 
 from geoh5py.objects import Grid2D, Points, Surface
 
 from geoapps.inversion import default_ui_json as base_default_ui_json
+from geoapps.inversion.constants import validations as base_validations
 
 ################# defaults ##################
 
 inversion_defaults = {
-    "title": "SimPEG Magnetic Vector inversion",
+    "title": "Magnetic Vector inversion",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "magnetic vector",
     "geoh5": None,  # Must remain at top of list for notebook app initialization
     "forward_only": False,
@@ -24,8 +29,21 @@ inversion_defaults = {
     "topography_object": None,
     "topography": None,
     "data_object": None,
+    "resolution": None,
+    "z_from_topo": False,
+    "receivers_offset_x": None,
+    "receivers_offset_y": None,
+    "receivers_offset_z": None,
+    "receivers_radar_drape": None,
+    "gps_receivers_offset": None,
     "tmi_channel": None,
     "tmi_uncertainty": 1.0,
+    "bx_channel": None,
+    "bx_uncertainty": 1.0,
+    "by_channel": None,
+    "by_uncertainty": 1.0,
+    "bz_channel": None,
+    "bz_uncertainty": 1.0,
     "bxx_channel": None,
     "bxx_uncertainty": 1.0,
     "bxy_channel": None,
@@ -38,42 +56,19 @@ inversion_defaults = {
     "byz_uncertainty": 1.0,
     "bzz_channel": None,
     "bzz_uncertainty": 1.0,
-    "bx_channel": None,
-    "bx_uncertainty": 1.0,
-    "by_channel": None,
-    "by_uncertainty": 1.0,
-    "bz_channel": None,
-    "bz_uncertainty": 1.0,
-    "starting_model_object": None,
-    "starting_inclination_object": None,
-    "starting_declination_object": None,
+    "mesh": None,
     "starting_model": 1e-4,
+    "reference_model": 0.0,
+    "lower_bound": None,
+    "upper_bound": None,
     "starting_inclination": None,
     "starting_declination": None,
-    "tile_spatial": 1,
+    "reference_inclination": None,
+    "reference_declination": None,
     "output_tile_files": False,
-    "z_from_topo": False,
-    "receivers_radar_drape": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
-    "receivers_offset_z": 0.0,
-    "gps_receivers_offset": None,
     "ignore_values": None,
-    "resolution": None,
     "detrend_order": None,
     "detrend_type": None,
-    "max_chunk_size": 128,
-    "chunk_by_rows": True,
-    "mesh": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "max_distance": 5000.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
     "window_center_x": None,
     "window_center_y": None,
     "window_width": None,
@@ -81,23 +76,13 @@ inversion_defaults = {
     "window_azimuth": None,
     "inversion_style": "voxel",
     "chi_factor": 1.0,
-    "sens_wts_threshold": 0.0,
-    "every_iteration_bool": False,
-    "f_min_change": 1e-4,
-    "minGNiter": 1,
-    "beta_tol": 0.5,
-    "prctile": 95,
+    "initial_beta_ratio": 10.0,
+    "initial_beta": None,
     "coolingRate": 1,
-    "coolEps_q": True,
-    "coolEpsFact": 1.2,
-    "beta_search": False,
-    "starting_chi_factor": None,
-    "max_iterations": 25,
+    "coolingFactor": 2.0,
+    "max_global_iterations": 100,
     "max_line_search_iterations": 20,
     "max_cg_iterations": 30,
-    "max_global_iterations": 100,
-    "initial_beta_ratio": 1e2,
-    "initial_beta": None,
     "tol_cg": 1e-4,
     "alpha_s": 1.0,
     "alpha_x": 1.0,
@@ -107,21 +92,25 @@ inversion_defaults = {
     "x_norm": 2.0,
     "y_norm": 2.0,
     "z_norm": 2.0,
-    "reference_model_object": None,
-    "reference_model": 0.0,
-    "reference_inclination_object": None,
-    "reference_declination_object": None,
-    "reference_inclination": None,
-    "reference_declination": None,
     "gradient_type": "total",
-    "lower_bound_object": None,
-    "lower_bound": None,
-    "upper_bound_object": None,
-    "upper_bound": None,
+    "max_irls_iterations": 25,
+    "starting_chi_factor": None,
+    "f_min_change": 1e-4,
+    "beta_tol": 0.5,
+    "prctile": 95,
+    "coolEps_q": True,
+    "coolEpsFact": 1.2,
+    "beta_search": False,
+    "sens_wts_threshold": 1.0,
+    "every_iteration_bool": False,
     "parallelized": True,
     "n_cpu": None,
+    "tile_spatial": 1,
+    "store_sensitivities": "disk",
     "max_ram": None,
-    "out_group": "VectorInversion",
+    "max_chunk_size": 128,
+    "chunk_by_rows": True,
+    "out_group": "MagneticVectorInversion",
     "monitoring_directory": None,
     "workspace_geoh5": None,
     "run_command": "geoapps.inversion.driver",
@@ -141,7 +130,9 @@ inversion_defaults = {
 }
 
 forward_defaults = {
-    "title": "SimPEG Magnetic Vector Forward",
+    "title": "Magnetic Vector Forward",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "magnetic vector",
     "geoh5": None,  # Must remain at top of list for notebook app initialization
     "forward_only": True,
@@ -151,43 +142,28 @@ forward_defaults = {
     "topography_object": None,
     "topography": None,
     "data_object": None,
+    "resolution": None,
+    "z_from_topo": False,
+    "receivers_offset_x": None,
+    "receivers_offset_y": None,
+    "receivers_offset_z": None,
+    "receivers_radar_drape": None,
+    "gps_receivers_offset": None,
     "tmi_channel_bool": False,
+    "bx_channel_bool": False,
+    "by_channel_bool": False,
+    "bz_channel_bool": False,
     "bxx_channel_bool": False,
     "bxy_channel_bool": False,
     "bxz_channel_bool": False,
     "byy_channel_bool": False,
     "byz_channel_bool": False,
     "bzz_channel_bool": False,
-    "bx_channel_bool": False,
-    "by_channel_bool": False,
-    "bz_channel_bool": False,
-    "starting_model_object": None,
-    "starting_inclination_object": None,
-    "starting_declination_object": None,
+    "mesh": None,
     "starting_model": None,
     "starting_inclination": None,
     "starting_declination": None,
-    "tile_spatial": 1,
     "output_tile_files": False,
-    "z_from_topo": False,
-    "receivers_radar_drape": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
-    "receivers_offset_z": 0.0,
-    "gps_receivers_offset": None,
-    "resolution": None,
-    "max_chunk_size": 128,
-    "chunk_by_rows": True,
-    "mesh": None,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
-    "octree_levels_topo": [0, 0, 4, 4],
-    "octree_levels_obs": [4, 4, 4, 4],
-    "depth_core": 500.0,
-    "max_distance": 5000.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
     "window_center_x": None,
     "window_center_y": None,
     "window_width": None,
@@ -195,7 +171,10 @@ forward_defaults = {
     "window_azimuth": None,
     "parallelized": True,
     "n_cpu": None,
-    "out_group": "MVIForward",
+    "tile_spatial": 1,
+    "max_chunk_size": 128,
+    "chunk_by_rows": True,
+    "out_group": "MagneticVectorForward",
     "monitoring_directory": None,
     "workspace_geoh5": None,
     "run_command": "geoapps.inversion.driver",
@@ -215,15 +194,15 @@ forward_defaults = {
 
 inversion_ui_json = {
     "tmi_channel_bool": False,
+    "bx_channel_bool": False,
+    "by_channel_bool": False,
+    "bz_channel_bool": False,
     "bxx_channel_bool": False,
     "bxy_channel_bool": False,
     "bxz_channel_bool": False,
     "byy_channel_bool": False,
     "byz_channel_bool": False,
     "bzz_channel_bool": False,
-    "bx_channel_bool": False,
-    "by_channel_bool": False,
-    "bz_channel_bool": False,
 }
 
 forward_ui_json = {
@@ -239,13 +218,15 @@ forward_ui_json = {
 }
 
 default_ui_json = {
-    "title": "SimPEG Magnetic Vector inversion",
+    "title": "Magnetic Vector inversion",
+    "documentation": "https://geoapps.readthedocs.io/en/stable/content/applications/grav_mag_inversion.html",
+    "icon": "surveyairbornegravity",
     "inversion_type": "magnetic vector",
     "inducing_field_strength": {
         "min": 0.0,
         "main": True,
         "group": "Inducing Field",
-        "label": "Strength",
+        "label": "Strength (nT)",
         "value": 50000.0,
     },
     "inducing_field_inclination": {
@@ -253,19 +234,35 @@ default_ui_json = {
         "max": 90.0,
         "main": True,
         "group": "Inducing Field",
-        "label": "Inclination",
+        "label": "Inclination (degrees from North)",
         "value": 90.0,
     },
     "inducing_field_declination": {
+        "min": -180,
+        "max": 180,
         "main": True,
         "group": "Inducing Field",
-        "label": "Declination",
+        "label": "Declination (degrees from horizontal)",
         "value": 0.0,
     },
+    "data_object": {
+        "main": True,
+        "group": "Data",
+        "label": "Object",
+        "meshType": [
+            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
+            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
+            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
+            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
+            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
+            "{4ea87376-3ece-438b-bf12-3479733ded46}",
+        ],
+        "value": None,
+    },
     "tmi_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use TMI",
+        "label": "TMI (nT)",
         "value": False,
     },
     "tmi_channel": {
@@ -273,10 +270,10 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "TMI channel",
+        "label": "TMI (nT)",
         "parent": "data_object",
         "optional": True,
-        "enabled": False,
+        "enabled": True,
         "value": None,
     },
     "tmi_uncertainty": {
@@ -285,7 +282,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "TMI uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "tmi_channel",
         "dependencyType": "enabled",
@@ -295,7 +292,7 @@ default_ui_json = {
     "bxx_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bxx",
+        "label": "Bxx (nT/m)",
         "value": False,
     },
     "bxx_channel": {
@@ -303,7 +300,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bxx channel",
+        "label": "Bxx (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -315,7 +312,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bxx uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bxx_channel",
         "dependencyType": "enabled",
@@ -325,7 +322,7 @@ default_ui_json = {
     "bxy_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bxy",
+        "label": "Bxy (nT/m)",
         "value": False,
     },
     "bxy_channel": {
@@ -333,7 +330,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bxy channel",
+        "label": "Bxy (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -345,7 +342,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bxy uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bxy_channel",
         "dependencyType": "enabled",
@@ -355,7 +352,7 @@ default_ui_json = {
     "bxz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bxz",
+        "label": "Bxz (nT/m)",
         "value": False,
     },
     "bxz_channel": {
@@ -363,7 +360,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bxz channel",
+        "label": "Bxz (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -375,7 +372,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bxz uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bxz_channel",
         "dependencyType": "enabled",
@@ -385,7 +382,7 @@ default_ui_json = {
     "byy_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Byy",
+        "label": "Byy (nT/m)",
         "value": False,
     },
     "byy_channel": {
@@ -393,7 +390,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Byy channel",
+        "label": "Byy (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -405,7 +402,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Byy uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "byy_channel",
         "dependencyType": "enabled",
@@ -415,7 +412,7 @@ default_ui_json = {
     "byz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Byz",
+        "label": "Byz (nT/m)",
         "value": False,
     },
     "byz_channel": {
@@ -423,7 +420,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Byz channel",
+        "label": "Byz (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -435,7 +432,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Byz uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "byz_channel",
         "dependencyType": "enabled",
@@ -445,7 +442,7 @@ default_ui_json = {
     "bzz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bzz",
+        "label": "Bzz (nT/m)",
         "value": False,
     },
     "bzz_channel": {
@@ -453,7 +450,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bzz channel",
+        "label": "Bzz (nT/m)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -465,7 +462,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bzz uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bzz_channel",
         "dependencyType": "enabled",
@@ -475,7 +472,7 @@ default_ui_json = {
     "bx_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bx",
+        "label": "Bx (nT)",
         "value": False,
     },
     "bx_channel": {
@@ -483,7 +480,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bx channel",
+        "label": "Bx (nT)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -495,7 +492,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bx uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bx_channel",
         "dependencyType": "enabled",
@@ -505,7 +502,7 @@ default_ui_json = {
     "by_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use By",
+        "label": "By (nT)",
         "value": False,
     },
     "by_channel": {
@@ -513,7 +510,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "By channel",
+        "label": "By (nT)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -525,7 +522,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "By uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "by_channel",
         "dependencyType": "enabled",
@@ -535,7 +532,7 @@ default_ui_json = {
     "bz_channel_bool": {
         "group": "Data",
         "main": True,
-        "label": "Use Bz",
+        "label": "Bz (nT)",
         "value": False,
     },
     "bz_channel": {
@@ -543,7 +540,7 @@ default_ui_json = {
         "dataType": "Float",
         "group": "Data",
         "main": True,
-        "label": "Bz channel",
+        "label": "Bz (nT)",
         "parent": "data_object",
         "optional": True,
         "enabled": False,
@@ -555,7 +552,7 @@ default_ui_json = {
         "group": "Data",
         "main": True,
         "isValue": True,
-        "label": "Bz uncertainty",
+        "label": "Uncertainty",
         "parent": "data_object",
         "dependency": "bz_channel",
         "dependencyType": "enabled",
@@ -565,123 +562,104 @@ default_ui_json = {
     "starting_model": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Starting Model",
+        "group": "Mesh and Models",
         "main": True,
         "isValue": False,
-        "parent": "starting_model_object",
-        "label": "Susceptibility (SI)",
+        "parent": "mesh",
+        "label": "Initial Susceptibility (SI)",
         "property": None,
-        "value": 0.0,
-    },
-    "starting_inclination_object": {
-        "group": "Starting Model",
-        "main": True,
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
-            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
-            "{4ea87376-3ece-438b-bf12-3479733ded46}",
-        ],
-        "label": "Starting inclination object",
-        "optional": True,
-        "value": None,
+        "value": 1e-4,
     },
     "starting_inclination": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Starting Model",
+        "group": "Mesh and Models",
         "main": True,
         "isValue": False,
         "optional": True,
         "enabled": False,
-        "parent": "starting_inclination_object",
-        "label": "Inclination (degree from North)",
+        "parent": "mesh",
+        "label": "Initial Inclination (degrees)",
         "property": None,
         "value": 0.0,
     },
-    "starting_declination_object": {
-        "group": "Starting Model",
-        "main": True,
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
-            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
-            "{4ea87376-3ece-438b-bf12-3479733ded46}",
-        ],
-        "label": "Starting declination object",
-        "optional": True,
-        "value": None,
-    },
     "starting_declination": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Starting Model",
+        "group": "Mesh and Models",
         "main": True,
         "isValue": False,
         "optional": True,
         "enabled": False,
-        "parent": "starting_declination_object",
-        "label": "Declination (degree from horizontal)",
+        "parent": "mesh",
+        "label": "Initial Declination (degrees)",
         "property": None,
         "value": 0.0,
     },
-    "reference_inclination_object": {
-        "group": "Regularization",
-        "label": "Reference inclination object",
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
-            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
-            "{4ea87376-3ece-438b-bf12-3479733ded46}",
-        ],
-        "optional": True,
-        "value": None,
+    "reference_model": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": True,
+        "parent": "mesh",
+        "label": "Reference Susceptibility (SI)",
+        "property": None,
+        "value": 0.0,
     },
     "reference_inclination": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Regularization",
+        "group": "Mesh and Models",
+        "main": True,
         "isValue": False,
         "optional": True,
         "enabled": False,
-        "label": "Reference inclination value",
-        "parent": "reference_inclination_object",
+        "label": "Reference Inclination (degrees)",
+        "parent": "mesh",
         "property": None,
         "value": 0.0,
     },
-    "reference_declination_object": {
-        "group": "Regularization",
-        "label": "Reference declination object",
-        "meshType": [
-            "{202C5DB1-A56D-4004-9CAD-BAAFD8899406}",
-            "{6A057FDC-B355-11E3-95BE-FD84A7FFCB88}",
-            "{F26FEBA3-ADED-494B-B9E9-B2BBCBE298E1}",
-            "{48F5054A-1C5C-4CA4-9048-80F36DC60A06}",
-            "{b020a277-90e2-4cd7-84d6-612ee3f25051}",
-            "{4ea87376-3ece-438b-bf12-3479733ded46}",
-        ],
-        "optional": True,
-        "value": None,
-    },
     "reference_declination": {
         "association": ["Cell", "Vertex"],
         "dataType": "Float",
-        "group": "Regularization",
+        "group": "Mesh and Models",
+        "main": True,
         "isValue": True,
         "optional": True,
         "enabled": False,
-        "label": "Reference declination value",
-        "parent": "reference_declination_object",
+        "label": "Reference Declination (degrees)",
+        "parent": "mesh",
         "property": None,
         "value": 0.0,
     },
-    "out_group": {"label": "Results group name", "value": "VectorInversion"},
+    "lower_bound": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": False,
+        "parent": "mesh",
+        "label": "Lower bound (SI)",
+        "property": None,
+        "optional": True,
+        "value": 0.0,
+        "enabled": False,
+    },
+    "upper_bound": {
+        "association": ["Cell", "Vertex"],
+        "main": True,
+        "dataType": "Float",
+        "group": "Mesh and Models",
+        "isValue": False,
+        "parent": "mesh",
+        "label": "Upper bound (SI)",
+        "property": None,
+        "optional": True,
+        "value": 1.0,
+        "enabled": False,
+    },
+    "out_group": {"label": "Results group name", "value": "MagneticVectorInversion"},
 }
 
 default_ui_json = dict(base_default_ui_json, **default_ui_json)
@@ -718,25 +696,20 @@ validations = {
     "bz_uncertainty": {"one_of": "uncertainty channel"},
 }
 
+validations = dict(base_validations, **validations)
+
 app_initializer = {
     "geoh5": "../../../assets/FlinFlon.geoh5",
     "forward_only": False,
     "data_object": UUID("{538a7eb1-2218-4bec-98cc-0a759aa0ef4f}"),
     "tmi_channel": UUID("{44822654-b6ae-45b0-8886-2d845f80f422}"),
     "tmi_uncertainty": 10.0,
+    "mesh": UUID("{385f341f-1027-4b8e-9a86-93be239aa3fb}"),
     "inducing_field_strength": 60000.0,
     "inducing_field_inclination": 79.0,
     "inducing_field_declination": 11.0,
     "reference_model": 0.0,
-    "u_cell_size": 25.0,
-    "v_cell_size": 25.0,
-    "w_cell_size": 25.0,
     "resolution": 50.0,
-    "octree_levels_topo": [0, 0, 0, 2],
-    "octree_levels_obs": [5, 5, 5, 5],
-    "depth_core": 500.0,
-    "horizontal_padding": 1000.0,
-    "vertical_padding": 1000.0,
     "window_center_x": 314600.0,
     "window_center_y": 6072300.0,
     "window_width": 1000.0,
@@ -746,20 +719,18 @@ app_initializer = {
     "x_norm": 2.0,
     "y_norm": 2.0,
     "z_norm": 2.0,
+    "lower_bound": None,
     "upper_bound": None,
     "starting_model": 1e-4,
     "starting_inclination": 79.0,
     "starting_declination": 11.0,
     "reference_inclination": None,
     "reference_declination": None,
-    "max_iterations": 25,
+    "max_global_iterations": 25,
     "topography_object": UUID("{ab3c2083-6ea8-4d31-9230-7aad3ec09525}"),
     "topography": UUID("{a603a762-f6cb-4b21-afda-3160e725bf7d}"),
     "z_from_topo": True,
     "detrend_order": 0,
     "detrend_type": None,
-    "receivers_offset_x": 0.0,
-    "receivers_offset_y": 0.0,
     "receivers_offset_z": 60.0,
-    "out_group": "VectorInversion",
 }
