@@ -5,52 +5,59 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 import sys
 import uuid
 import warnings
 from copy import deepcopy
 from os import path
-from typing import Optional
+from time import time
 
-import matplotlib.pyplot as plt
 import numpy as np
 from geoh5py.data import ReferencedData
 from geoh5py.objects import Curve, ObjectBase
 from geoh5py.shared import Entity
 from geoh5py.ui_json import InputFile
 from geoh5py.workspace import Workspace
-from ipywidgets import (
-    Box,
-    Checkbox,
-    ColorPicker,
-    Dropdown,
-    FloatLogSlider,
-    FloatSlider,
-    FloatText,
-    HBox,
-    IntSlider,
-    Label,
-    Layout,
-    ToggleButton,
-    ToggleButtons,
-    VBox,
-    Widget,
-    interactive_output,
-)
-from ipywidgets.widgets.widget_selection import TraitError
 
+from geoapps.base.application import BaseApplication
 from geoapps.base.selection import LineOptions, ObjectDataSelection
 from geoapps.peak_finder.constants import (
     app_initializer,
     default_ui_json,
     template_dict,
 )
-from geoapps.utils import geophysical_systems
+from geoapps.utils import geophysical_systems, warn_module_not_found
 
 from . import PeakFinderParams
 from .driver import PeakFinderDriver
 from .utils import default_groups_from_property_group, find_anomalies
 
+with warn_module_not_found():
+    from matplotlib import pyplot as plt
+
+with warn_module_not_found():
+    from ipywidgets import (
+        Box,
+        Checkbox,
+        ColorPicker,
+        Dropdown,
+        FloatLogSlider,
+        FloatSlider,
+        FloatText,
+        HBox,
+        IntSlider,
+        Label,
+        Layout,
+        ToggleButton,
+        ToggleButtons,
+        VBox,
+        Widget,
+        interactive_output,
+    )
+    from ipywidgets.widgets.widget_selection import TraitError
+
 
 class PeakFinder(ObjectDataSelection):
     """
@@ -82,6 +89,7 @@ class PeakFinder(ObjectDataSelection):
     _system = None
     _tem_checkbox = None
     _width = None
+    _x_label = None
     _object_types = (Curve,)
     all_anomalies = []
     active_channels = {}
@@ -93,6 +101,7 @@ class PeakFinder(ObjectDataSelection):
     plot_result = True
 
     def __init__(self, ui_json=None, plot_result=True, **kwargs):
+        self.figure = None
         self.plot_result = plot_result
         app_initializer.update(kwargs)
         if ui_json is not None and path.exists(ui_json):
@@ -100,7 +109,6 @@ class PeakFinder(ObjectDataSelection):
         else:
             self.params = self._param_class(**app_initializer)
 
-        self.defaults = {}
         for key, value in self.params.to_dict().items():
             if isinstance(value, Entity):
                 self.defaults[key] = value.uid
@@ -202,7 +210,7 @@ class PeakFinder(ObjectDataSelection):
 
         obj_list = self.workspace.get_entity(self.objects.value)
 
-        if any(obj_list) and any(self.params.free_parameter_dict):
+        if obj_list[0] is not None and any(self.params.free_parameter_dict):
             self._channel_groups = self.params.groups_from_free_params()
 
             group_list = []
@@ -430,7 +438,7 @@ class PeakFinder(ObjectDataSelection):
                 continuous_update=False,
                 description="Minimum # channels",
                 style={"description_width": "initial"},
-                disabled=True,
+                disabled=False,
             )
 
         return self._min_channels
@@ -563,7 +571,7 @@ class PeakFinder(ObjectDataSelection):
         return self._structural_markers
 
     @property
-    def survey(self) -> Optional[Entity]:
+    def survey(self) -> Entity | None:
         """
         Selected curve object
         """
@@ -708,6 +716,9 @@ class PeakFinder(ObjectDataSelection):
     def create_default_groups(self, _):
         if self.group_auto.value:
             obj = self.workspace.get_entity(self.objects.value)[0]
+            if obj is None:
+                return
+
             group = [pg for pg in obj.property_groups if pg.uid == self.data.value]
             if any(group):
                 channel_groups = default_groups_from_property_group(group[0])
@@ -757,15 +768,8 @@ class PeakFinder(ObjectDataSelection):
         """
         line_data = self.workspace.get_entity(self.lines.data.value)[0]
 
-        if isinstance(line_data, ReferencedData):
-            line_id = [
-                key
-                for key, value in line_data.value_map.map.items()
-                if value == line_id
-            ]
-
-            if line_id:
-                line_id = line_id[0]
+        if not isinstance(line_data, ReferencedData):
+            return
 
         indices = np.where(np.asarray(line_data.values) == line_id)[0]
 
@@ -861,33 +865,33 @@ class PeakFinder(ObjectDataSelection):
         """
         Observer of :obj:`geoapps.processing.peak_finder.objects`: Reset data and auto-detect AEM system
         """
-        if self.workspace.get_entity(self.objects.value):
-            self._survey = self.workspace.get_entity(self.objects.value)[0]
-            self.update_data_list(None)
-            is_tem = False
-            self.active_channels = {}
-            self.channel_groups = {}
-            for child in self.groups_panel.children:
-                child.children[0].options = self.data.options
+        obj = self.workspace.get_entity(self.objects.value)[0]
+        if obj is None:
+            return
 
-            for aem_system, specs in self.em_system_specs.items():
-                if any(
-                    [
-                        specs["flag"] in channel
-                        for channel in self._survey.get_data_list()
-                    ]
-                ):
-                    if aem_system in self.system.options:
-                        self.system.value = aem_system
-                        is_tem = True
-                        break
+        self._survey = obj
+        self.update_data_list(None)
+        is_tem = False
+        self.active_channels = {}
+        self.channel_groups = {}
+        for child in self.groups_panel.children:
+            child.children[0].options = self.data.options
 
-            self.tem_checkbox.value = is_tem
+        for aem_system, specs in self.em_system_specs.items():
+            if specs["flag"] is not None and any(
+                [specs["flag"] in channel for channel in self._survey.get_data_list()]
+            ):
+                if aem_system in self.system.options:
+                    self.system.value = aem_system
+                    is_tem = True
+                    break
 
-            if self.group_auto:
-                self.create_default_groups(None)
+        self.tem_checkbox.value = is_tem
 
-            self.set_data(None)
+        if self.group_auto:
+            self.create_default_groups(None)
+
+        self.set_data(None)
 
     def tem_change(self, _):
         self.min_channels.disabled = not self.tem_checkbox.value
@@ -912,8 +916,8 @@ class PeakFinder(ObjectDataSelection):
 
         if (
             self.pause_refresh
-            or not self.refresh.value
-            or self.plot_trigger.value is False
+            or not refresh
+            or plot_trigger is False
             or not self.plot_result
         ):
             return
@@ -948,15 +952,15 @@ class PeakFinder(ObjectDataSelection):
         up_markers_x, up_markers_y = [], []
         dwn_markers_x, dwn_markers_y = [], []
 
-        for cc, (uid, channel) in enumerate(self.active_channels.items()):
+        for cc, channel in enumerate(self.active_channels.values()):
 
-            if "values" not in channel.keys():
+            if "values" not in channel:
                 continue
 
             self.lines.profile.values = channel["values"][self.survey.line_indices]
             values = self.lines.profile.values_resampled
-            y_min = np.min([values[sub_ind].min(), y_min])
-            y_max = np.max([values[sub_ind].max(), y_max])
+            y_min = np.nanmin([values[sub_ind].min(), y_min])
+            y_max = np.nanmax([values[sub_ind].max(), y_max])
             axs.plot(locs, values, color=[0.5, 0.5, 0.5, 1])
             for group in self.lines.anomalies:
                 query = np.where(group["channels"] == cc)[0]
@@ -968,9 +972,9 @@ class PeakFinder(ObjectDataSelection):
                 ):
                     continue
 
-                ii = query[0]
-                start = group["start"][ii]
-                end = group["end"][ii]
+                i = query[0]
+                start = group["start"][i]
+                end = group["end"][i]
                 axs.plot(
                     locs[start:end],
                     values[start:end],
@@ -983,29 +987,29 @@ class PeakFinder(ObjectDataSelection):
                     ori = "left"
 
                 if markers:
-                    if ii == 0:
+                    if i == 0:
                         axs.scatter(
-                            locs[group["peak"][ii]],
-                            values[group["peak"][ii]],
+                            locs[group["peak"][i]],
+                            values[group["peak"][i]],
                             s=200,
                             c="k",
                             marker=self.marker[ori],
                             zorder=10,
                         )
-                    peak_markers_x += [locs[group["peak"][ii]]]
-                    peak_markers_y += [values[group["peak"][ii]]]
+                    peak_markers_x += [locs[group["peak"][i]]]
+                    peak_markers_y += [values[group["peak"][i]]]
                     peak_markers_c += [group["channel_group"]["color"]]
-                    start_markers_x += [locs[group["start"][ii]]]
-                    start_markers_y += [values[group["start"][ii]]]
-                    end_markers_x += [locs[group["end"][ii]]]
-                    end_markers_y += [values[group["end"][ii]]]
-                    up_markers_x += [locs[group["inflx_up"][ii]]]
-                    up_markers_y += [values[group["inflx_up"][ii]]]
-                    dwn_markers_x += [locs[group["inflx_dwn"][ii]]]
-                    dwn_markers_y += [values[group["inflx_dwn"][ii]]]
+                    start_markers_x += [locs[group["start"][i]]]
+                    start_markers_y += [values[group["start"][i]]]
+                    end_markers_x += [locs[group["end"][i]]]
+                    end_markers_y += [values[group["end"][i]]]
+                    up_markers_x += [locs[group["inflx_up"][i]]]
+                    up_markers_y += [values[group["inflx_up"][i]]]
+                    dwn_markers_x += [locs[group["inflx_dwn"][i]]]
+                    dwn_markers_y += [values[group["inflx_dwn"][i]]]
 
             if residual:
-                raw = self.lines.profile._values_resampled_raw
+                raw = self.lines.profile.values_resampled_raw
                 axs.fill_between(
                     locs, values, raw, where=raw > values, color=[1, 0, 0, 0.5]
                 )
@@ -1023,7 +1027,7 @@ class PeakFinder(ObjectDataSelection):
             center - width / 2.0,
             center + width / 2.0,
         ]
-        y_lims = [np.max([y_min, self.min_value.value]), y_max]
+        y_lims = [np.nanmax([y_min, self.min_value.value]), y_max]
         axs.set_xlim(x_lims)
         axs.set_ylim(y_lims)
         axs.set_ylabel("Data")
@@ -1118,7 +1122,7 @@ class PeakFinder(ObjectDataSelection):
             return
 
         if (
-            self.plot_trigger.value
+            plot_trigger
             or self.refresh.value
             and hasattr(self.lines, "profile")
             and self.tem_checkbox.value
@@ -1149,8 +1153,8 @@ class PeakFinder(ObjectDataSelection):
             if group is not None and group["linear_fit"] is not None:
                 times = [
                     channel["time"]
-                    for ii, channel in enumerate(self.active_channels.values())
-                    if ii in list(group["channels"])
+                    for i, channel in enumerate(self.active_channels.values())
+                    if i in list(group["channels"])
                 ]
             if any(times):
                 times = np.hstack(times)
@@ -1226,9 +1230,7 @@ class PeakFinder(ObjectDataSelection):
                 try:
                     if self.tem_checkbox.value:
                         channel = [
-                            ch
-                            for ch in system["channels"].keys()
-                            if ch in params["name"]
+                            ch for ch in system["channels"] if ch in params["name"]
                         ]
                         if any(channel):
                             self.active_channels[uid]["time"] = system["channels"][
@@ -1248,8 +1250,12 @@ class PeakFinder(ObjectDataSelection):
                             ),
                         ]
                     )
-                    d_min = np.min([d_min, self.active_channels[uid]["values"].min()])
-                    d_max = np.max([d_max, self.active_channels[uid]["values"].max()])
+                    d_min = np.nanmin(
+                        [d_min, self.active_channels[uid]["values"].min()]
+                    )
+                    d_max = np.nanmax(
+                        [d_max, self.active_channels[uid]["values"].max()]
+                    )
                 except KeyError:
                     continue
 
@@ -1284,42 +1290,47 @@ class PeakFinder(ObjectDataSelection):
 
         for label, group in self._channel_groups.items():
             for member in ["data", "color"]:
-                name = f"{label} {member}"
+                name = f"Group {label} {member}"
                 ui_json[name] = deepcopy(template_dict[member])
                 ui_json[name]["group"] = f"Group {label}"
                 param_dict[name] = group[member]
 
-        new_workspace = self.get_output_workspace(
-            self.export_directory.selected_path, self.ga_group_name.value
+        p_g_uid = {p_g.uid: p_g.name for p_g in param_dict["objects"].property_groups}
+
+        temp_geoh5 = f"{self.ga_group_name.value}_{time():.0f}.geoh5"
+
+        ws, self.live_link.value = BaseApplication.get_output_workspace(
+            self.live_link.value, self.export_directory.selected_path, temp_geoh5
         )
-        for key, value in param_dict.items():
-            if isinstance(value, ObjectBase):
-                if new_workspace.get_entity(value.uid)[0] is None:
-                    param_dict[key] = value.copy(
-                        parent=new_workspace, copy_children=True
-                    )
-                    line_field = [
-                        c for c in param_dict[key].children if c.name == "Line"
-                    ]
-                    if line_field:
-                        param_dict["line_field"] = line_field[0]
+        with ws as new_workspace:
 
-        param_dict["geoh5"] = new_workspace
-        if self.live_link.value:
-            param_dict["monitoring_directory"] = self.monitoring_directory
+            for key, value in param_dict.items():
+                if isinstance(value, ObjectBase):
+                    if new_workspace.get_entity(value.uid)[0] is None:
+                        param_dict[key] = value.copy(
+                            parent=new_workspace, copy_children=True
+                        )
+                        line_field = [
+                            c for c in param_dict[key].children if c.name == "Line"
+                        ]
+                        if line_field:
+                            param_dict["line_field"] = line_field[0]
 
-        ifile = InputFile(
-            ui_json=ui_json,
-            validations=self.params.validations,
-            validation_options={"disabled": True},
-        )
+                elif isinstance(value, uuid.UUID) and value in p_g_uid:
+                    param_dict[key] = param_dict[
+                        "objects"
+                    ].find_or_create_property_group(name=p_g_uid[value])
 
-        new_params = PeakFinderParams(input_file=ifile, **param_dict)
-        new_params.write_input_file()
-        self.run(new_params)
+            param_dict["geoh5"] = new_workspace
+            if self.live_link.value:
+                param_dict["monitoring_directory"] = self.monitoring_directory
 
-        if self.live_link.value:
-            print("Live link active. Check your ANALYST session for result.")
+            new_params = PeakFinderParams(**param_dict)
+            new_params.write_input_file(name=temp_geoh5.replace(".geoh5", ".ui.json"))
+            self.run(new_params)
+
+            if self.live_link.value:
+                print("Live link active. Check your ANALYST session for result.")
 
     def update_center(self, _):
         """
@@ -1328,14 +1339,14 @@ class PeakFinder(ObjectDataSelection):
         if hasattr(self.lines, "anomalies"):
             self.center.value = self.group_display.value
 
-    @staticmethod
-    def run(params, output_group=None):
+    @classmethod
+    def run(cls, params: PeakFinderParams):
         """
         Create an octree mesh from input values
         """
-
         driver = PeakFinderDriver(params)
-        driver.run(output_group)
+        with params.geoh5.open(mode="r+"):
+            driver.run()
 
     def show_decay_trigger(self, _):
         """
@@ -1356,5 +1367,5 @@ if __name__ == "__main__":
         "'geoapps.peak_finder.driver' in version 0.7.0. "
         "This warning is likely due to the execution of older ui.json files. Please update."
     )
-    params = PeakFinderParams(InputFile(file))
-    PeakFinder.run(params)
+    params_class = PeakFinderParams(InputFile(file))
+    PeakFinder.run(params_class)
