@@ -5,4 +5,7 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
-from .application import ScatterPlots
+from __future__ import annotations
+
+from .constants import app_initializer
+from .params import ScatterPlotParams
