@@ -5,20 +5,62 @@
 #  geoapps is distributed under the terms and conditions of the MIT License
 #  (see LICENSE file at the root of this source code package).
 
+from __future__ import annotations
+
 from copy import copy
 
-import matplotlib.colors as colors
-import matplotlib.pyplot as plt
 import numpy as np
-import plotly.graph_objects as go
-from geoh5py.data import Data, ReferencedData
-from geoh5py.groups import ContainerGroup
+from geoh5py.data import Data
+from geoh5py.groups import SimPEGGroup
 from geoh5py.objects import BlockModel, Curve, Grid2D, Points, Surface
 from geoh5py.workspace import Workspace
-from ipywidgets import widgets
 
-from geoapps.utils import get_inversion_output
-from geoapps.utils.utils import filter_xy, format_labels, inv_symlog, symlog
+from geoapps.shared_utils.utils import filter_xy, get_inversion_output
+from geoapps.utils import warn_module_not_found
+
+with warn_module_not_found():
+    from matplotlib import colors
+    from matplotlib import pyplot as plt
+
+with warn_module_not_found():
+    from plotly import graph_objects as go
+
+with warn_module_not_found():
+    import ipywidgets as widgets
+
+
+def symlog(values, threshold):
+    """
+    Convert values to log with linear threshold near zero
+    """
+    return np.sign(values) * np.log10(1 + np.abs(values) / threshold)
+
+
+def inv_symlog(values, threshold):
+    """
+    Compute the inverse symlog mapping
+    """
+    return np.sign(values) * threshold * (-1.0 + 10.0 ** np.abs(values))
+
+
+def format_labels(x, y, axs, labels=None, aspect="equal", tick_format="%i"):
+    if labels is None:
+        axs.set_ylabel("Northing (m)")
+        axs.set_xlabel("Easting (m)")
+    else:
+        axs.set_xlabel(labels[0])
+        axs.set_ylabel(labels[1])
+    xticks = np.linspace(x.min(), x.max(), 5)
+    yticks = np.linspace(y.min(), y.max(), 5)
+
+    axs.set_yticks(yticks)
+    axs.set_yticklabels(
+        [tick_format % y for y in yticks.tolist()], rotation=90, va="center"
+    )
+    axs.set_xticks(xticks)
+    axs.set_xticklabels([tick_format % x for x in xticks.tolist()], va="center")
+    axs.autoscale(tight=True)
+    axs.set_aspect(aspect)
 
 
 def normalize(values):
@@ -76,7 +118,7 @@ def plot_plan_data_selection(entity, data, **kwargs):
     out = None
 
     if isinstance(entity, (Grid2D, Points, Curve, Surface)):
-        if "axis" not in kwargs.keys():
+        if "axis" not in kwargs:
             plt.figure(figsize=(8, 8))
             axis = plt.subplot()
         else:
@@ -92,12 +134,12 @@ def plot_plan_data_selection(entity, data, **kwargs):
     else:
         locations = entity.centroids
 
-    if "resolution" not in kwargs.keys():
+    if "resolution" not in kwargs:
         resolution = 0
     else:
         resolution = kwargs["resolution"]
 
-    if "indices" in kwargs.keys():
+    if "indices" in kwargs:
         indices = kwargs["indices"]
         if isinstance(indices, np.ndarray) and np.all(indices == False):
             indices = None
@@ -114,11 +156,11 @@ def plot_plan_data_selection(entity, data, **kwargs):
         values = None
 
     color_norm = None
-    if "color_norm" in kwargs.keys():
+    if "color_norm" in kwargs:
         color_norm = kwargs["color_norm"]
 
     window = None
-    if "window" in kwargs.keys():
+    if "window" in kwargs:
         window = kwargs["window"]
 
     if (
@@ -126,13 +168,13 @@ def plot_plan_data_selection(entity, data, **kwargs):
         and getattr(data, "entity_type", None) is not None
         and getattr(data.entity_type, "color_map", None) is not None
     ):
-        new_cmap = data.entity_type.color_map._values
-        map_vals = new_cmap["Value"].copy()
+        new_cmap = data.entity_type.color_map.values
+        map_vals = new_cmap[0].copy()
         cmap = colors.ListedColormap(
             np.c_[
-                new_cmap["Red"] / 255,
-                new_cmap["Green"] / 255,
-                new_cmap["Blue"] / 255,
+                new_cmap[1] / 255,
+                new_cmap[2] / 255,
+                new_cmap[3] / 255,
             ]
         )
         color_norm = colors.BoundaryNorm(map_vals, cmap.N)
@@ -162,11 +204,7 @@ def plot_plan_data_selection(entity, data, **kwargs):
                 X, Y, values, cmap=cmap, norm=color_norm, shading="auto"
             )
 
-        if (
-            "contours" in kwargs.keys()
-            and kwargs["contours"] is not None
-            and np.any(values)
-        ):
+        if "contours" in kwargs and kwargs["contours"] is not None and np.any(values):
             contour_set = axis.contour(
                 X, Y, values, levels=kwargs["contours"], colors="k", linewidths=1.0
             )
@@ -185,18 +223,14 @@ def plot_plan_data_selection(entity, data, **kwargs):
         if values is not None:
             values = values[indices]
 
-        if "marker_size" not in kwargs.keys():
+        if "marker_size" not in kwargs:
             marker_size = 50
         else:
             marker_size = kwargs["marker_size"]
 
         out = axis.scatter(X, Y, marker_size, values, cmap=cmap, norm=color_norm)
 
-        if (
-            "contours" in kwargs.keys()
-            and kwargs["contours"] is not None
-            and np.any(values)
-        ):
+        if "contours" in kwargs and kwargs["contours"] is not None and np.any(values):
             ind = ~np.isnan(values)
             contour_set = axis.tricontour(
                 X[ind],
@@ -207,11 +241,11 @@ def plot_plan_data_selection(entity, data, **kwargs):
                 linewidths=1.0,
             )
 
-    if "collections" in kwargs.keys():
+    if "collections" in kwargs:
         for collection in kwargs["collections"]:
             axis.add_collection(copy(collection))
 
-    if "zoom_extent" in kwargs.keys() and kwargs["zoom_extent"] and np.any(values):
+    if "zoom_extent" in kwargs and kwargs["zoom_extent"] and np.any(values):
         ind = ~np.isnan(values.ravel())
         x = X.ravel()[ind]
         y = Y.ravel()[ind]
@@ -219,16 +253,22 @@ def plot_plan_data_selection(entity, data, **kwargs):
     if np.any(x) and np.any(y):
         width = x.max() - x.min()
         height = y.max() - y.min()
-
-        format_labels(x, y, axis, **kwargs)
+        format_labels(
+            x,
+            y,
+            axis,
+            labels=kwargs.get("labels"),
+            aspect=kwargs.get("aspect", "equal"),
+            tick_format=kwargs.get("tick_format", "%i"),
+        )
         axis.set_xlim([x.min() - width * 0.1, x.max() + width * 0.1])
         axis.set_ylim([y.min() - height * 0.1, y.max() + height * 0.1])
 
-    if "colorbar" in kwargs.keys() and kwargs["colorbar"]:
+    if "colorbar" in kwargs and kwargs["colorbar"]:
         plt.colorbar(out, ax=axis)
 
     line_selection = np.zeros_like(indices, dtype=bool)
-    if "highlight_selection" in kwargs.keys() and isinstance(
+    if "highlight_selection" in kwargs and isinstance(
         kwargs["highlight_selection"], dict
     ):
         for key, values in kwargs["highlight_selection"].items():
@@ -237,12 +277,6 @@ def plot_plan_data_selection(entity, data, **kwargs):
                 continue
 
             line_data = entity.workspace.get_entity(key)[0]
-            if isinstance(line_data, ReferencedData):
-                values = [
-                    key
-                    for key, value in line_data.value_map.map.items()
-                    if value in values
-                ]
 
             for line in values:
                 ind = np.where(line_data.values == line)[0]
@@ -262,22 +296,25 @@ def plot_profile_data_selection(
     entity,
     field_list,
     uncertainties=None,
-    selection={},
+    selection=None,
     resolution=None,
     plot_legend=False,
     ax=None,
-    color=[0, 0, 0],
+    color=(0, 0, 0),
 ):
 
     locations = entity.vertices
 
     if ax is None:
-        fig = plt.figure(figsize=(8, 8))
+        plt.figure(figsize=(8, 8))
         ax = plt.subplot()
 
-    pos = ax.get_position()
     xx, yy = [], []
     threshold = 1e-14
+
+    if selection is None:
+        return ax, threshold
+
     for key, values in selection.items():
 
         for line in values:
@@ -311,7 +348,7 @@ def plot_profile_data_selection(
 
             c_increment = [(1 - c) / (len(field_list) + 1) for c in color]
 
-            for ii, field in enumerate(field_list):
+            for i, field in enumerate(field_list):
                 if (
                     entity.workspace.get_entity(field)
                     and entity.workspace.get_entity(field)[0].values is not None
@@ -325,15 +362,15 @@ def plot_profile_data_selection(
                         ax.errorbar(
                             xx[-1],
                             yy[-1],
-                            yerr=uncertainties[ii][0] * np.abs(yy[-1])
-                            + uncertainties[ii][1],
-                            color=[c + ii * i for c, i in zip(color, c_increment)],
+                            yerr=uncertainties[i][0] * np.abs(yy[-1])
+                            + uncertainties[i][1],
+                            color=[c + i * i for c, i in zip(color, c_increment)],
                         )
                     else:
                         ax.plot(
                             xx[-1],
                             yy[-1],
-                            color=[c + ii * i for c, i in zip(color, c_increment)],
+                            color=[c + i * i for c, i in zip(color, c_increment)],
                         )
                     legend.append(field)
 
@@ -368,7 +405,7 @@ def plotly_scatter(
     """
     assert (
         getattr(points, "vertices", None) is not None
-    ), f"Input object must have vertices"
+    ), "Input object must have vertices"
 
     if figure is None:
         figure = go.FigureWidget()
@@ -438,9 +475,9 @@ def plotly_block_model(
     block_model,
     figure=None,
     value=None,
-    x_slice=[],
-    y_slice=[],
-    z_slice=[],
+    x_slice=None,
+    y_slice=None,
+    z_slice=None,
     colorscale="Portland",
     **kwargs,
 ):
@@ -454,13 +491,13 @@ def plotly_block_model(
     if figure is None:
         figure = go.FigureWidget()
 
-    if not x_slice:
+    if x_slice is None:
         x_slice = [block_model.centroids[:, 0].mean()]
 
-    if not y_slice:
+    if y_slice is None:
         y_slice = [block_model.centroids[:, 1].mean()]
 
-    if not z_slice:
+    if z_slice is None:
         z_slice = [block_model.centroids[:, 2].mean()]
 
     figure.add_trace(go.Volume())
@@ -504,147 +541,10 @@ def check_data_type(data):
     return data
 
 
-# def plot_em_data_widget(h5file):
-#     workspace = Workspace(h5file)
-#
-#     curves = [
-#         entity.parent.name + "." + entity.name
-#         for entity in workspace.objects
-#         if isinstance(entity, Curve)
-#     ]
-#     names = [name for name in sorted(curves)]
-#
-#     def get_parental_child(parental_name):
-#
-#         parent, child = parental_name.split(".")
-#
-#         parent_entity = workspace.get_entity(parent)[0]
-#
-#         children = [entity for entity in parent_entity.children if entity.name == child]
-#         return children
-#
-#     def plot_profiles(entity_name, groups, line_field, lines, scale, threshold):
-#
-#         fig = plt.figure(figsize=(12, 12))
-#         entity = get_parental_child(entity_name)[0]
-#
-#         ax = plt.subplot()
-#         colors = [[0, 0, 0], [1, 0, 0], [0, 1, 0], [0, 0, 1]]
-#
-#         for group, color in zip(groups, colors):
-#
-#             prop_group = entity.get_property_group(group)
-#
-#             if prop_group is not None:
-#                 fields = [
-#                     entity.workspace.get_entity(uid)[0].name
-#                     for uid in prop_group.properties
-#                 ]
-#
-#                 ax, _ = plot_profile_data_selection(
-#                     prop_group.parent,
-#                     fields,
-#                     selection={line_field: lines},
-#                     ax=ax,
-#                     color=color,
-#                 )
-#
-#         ax.grid(True)
-#
-#         plt.yscale(scale, linthreshy=10.0 ** threshold)
-#
-#     def updateList(_):
-#         entity = get_parental_child(objects.value)[0]
-#         data_list = entity.get_data_list()
-#         obj = get_parental_child(objects.value)[0]
-#
-#         options = [pg.name for pg in obj.property_groups]
-#         options = [option for option in sorted(options)]
-#         groups.options = options
-#         groups.value = [groups.options[0]]
-#         line_field.options = data_list
-#         line_field.value = find_value(data_list, ["line"])
-#
-#         if line_field.value is None:
-#             line_ids = []
-#             value = []
-#         else:
-#             line_ids = np.unique(entity.get_data(line_field.value)[0].values)
-#             value = [line_ids[0]]
-#
-#         lines.options = line_ids
-#         lines.value = value
-#
-#     objects = Dropdown(options=names, value=names[0], description="Object:",)
-#
-#     obj = get_parental_child(objects.value)[0]
-#
-#     order = np.sort(obj.vertices[:, 0])
-#
-#     entity = get_parental_child(objects.value)[0]
-#
-#     data_list = entity.get_data_list()
-#     line_field = Dropdown(
-#         options=data_list,
-#         value=find_value(data_list, ["line"]),
-#         description="Lines field",
-#     )
-#
-#     options = [pg.name for pg in obj.property_groups]
-#     options = [option for option in sorted(options)]
-#     groups = SelectMultiple(options=options, value=[options[0]], description="Data: ",)
-#
-#     if line_field.value is None:
-#         line_list = []
-#         value = []
-#     else:
-#
-#         line_list = np.unique(entity.get_data(line_field.value)[0].values)
-#         value = [line_list[0]]
-#
-#     lines = SelectMultiple(options=line_list, value=value, description="Data: ")
-#
-#     objects.observe(updateList, names="value")
-#
-#     scale = Dropdown(
-#         options=["linear", "symlog"], value="symlog", description="Scaling",
-#     )
-#
-#     threshold = FloatSlider(
-#         min=-16,
-#         max=-1,
-#         value=-12,
-#         steps=0.5,
-#         description="Log-linear threshold",
-#         continuous_update=False,
-#     )
-#
-#     apps = VBox([objects, line_field, lines, groups, scale, threshold])
-#     layout = HBox(
-#         [
-#             apps,
-#             interactive_output(
-#                 plot_profiles,
-#                 {
-#                     "entity_name": objects,
-#                     "groups": groups,
-#                     "line_field": line_field,
-#                     "lines": lines,
-#                     "scale": scale,
-#                     "threshold": threshold,
-#                 },
-#             ),
-#         ]
-#     )
-#     return layout
-
-
 def plot_convergence_curve(h5file):
     """"""
     workspace = Workspace(h5file)
-    names = [
-        group.name for group in workspace.groups if isinstance(group, ContainerGroup)
-    ]
+    names = [group.name for group in workspace.groups if isinstance(group, SimPEGGroup)]
     objects = widgets.Dropdown(
         options=names,
         value=names[0],
@@ -656,20 +556,20 @@ def plot_convergence_curve(h5file):
 
         inversion = workspace.get_entity(objects)[0]
         result = None
-        if getattr(inversion, "comments", None) is not None:
-            if inversion.comments.values is not None:
-                result = get_inversion_output(workspace.h5file, objects)
-                iterations = result["iteration"]
-                phi_d = result["phi_d"]
-                phi_m = result["phi_m"]
+        child_names = [k.name for k in inversion.children]
+        if "SimPEG.out" in child_names:
+            result = get_inversion_output(workspace.h5file, objects)
+            iterations = result["iteration"]
+            phi_d = result["phi_d"]
+            phi_m = result["phi_m"]
 
-                ax1 = plt.subplot()
-                ax2 = ax1.twinx()
-                ax1.plot(iterations, phi_d, linewidth=3, c="k")
-                ax1.set_xlabel("Iterations")
-                ax1.set_ylabel(r"$\phi_d$", size=16)
-                ax2.plot(iterations, phi_m, linewidth=3, c="r")
-                ax2.set_ylabel(r"$\phi_m$", size=16)
+            ax1 = plt.subplot()
+            ax2 = ax1.twinx()
+            ax1.plot(iterations, phi_d, linewidth=3, c="k")
+            ax1.set_xlabel("Iterations")
+            ax1.set_ylabel(r"$\phi_d$", size=16)
+            ax2.plot(iterations, phi_m, linewidth=3, c="r")
+            ax2.set_ylabel(r"$\phi_m$", size=16)
 
         return result
 
